package net.minecraft.client;

import java.io.Writer;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.FileReader;
import org.lwjgl.input.Keyboard;
import java.io.File;

public final class GameSettings {
    private static final String[] RENDER_DISTANCES;
    private static final String[] DIFFICULTIES;
    public boolean music;
    public boolean sound;
    public boolean invertMouse;
    public boolean showFPS;
    public int renderDistance;
    public boolean fancyGraphics;
    public boolean anaglyph;
    public boolean limitFramerate;
    public KeyBinding keyBindForward;
    public KeyBinding keyBindLeft;
    public KeyBinding keyBindBack;
    public KeyBinding keyBindRight;
    public KeyBinding keyBindJump;
    public KeyBinding keyBindInventory;
    public KeyBinding keyBindDrop;
    private KeyBinding keyBindChat;
    public KeyBinding keyBindToggleFog;
    public KeyBinding keyBindSave;
    public KeyBinding keyBindLoad;
    public KeyBinding[] keyBindings;
    private Minecraft mc;
    private File optionsFile;
    public int numberOfOptions;
    public int difficulty;
    public boolean thirdPersonView;
    
    public GameSettings(final Minecraft minecraft, final File file) {
        this.music = true;
        this.sound = true;
        this.invertMouse = false;
        this.showFPS = false;
        this.renderDistance = 0;
        this.fancyGraphics = true;
        this.anaglyph = false;
        this.limitFramerate = false;
        this.keyBindForward = new KeyBinding("Forward", 17);
        this.keyBindLeft = new KeyBinding("Left", 30);
        this.keyBindBack = new KeyBinding("Back", 31);
        this.keyBindRight = new KeyBinding("Right", 32);
        this.keyBindJump = new KeyBinding("Jump", 57);
        this.keyBindInventory = new KeyBinding("Inventory", 23);
        this.keyBindDrop = new KeyBinding("Drop", 16);
        this.keyBindChat = new KeyBinding("Chat", 20);
        this.keyBindToggleFog = new KeyBinding("Toggle fog", 33);
        this.keyBindSave = new KeyBinding("Save location", 28);
        this.keyBindLoad = new KeyBinding("Load location", 19);
        this.keyBindings = new KeyBinding[] { this.keyBindForward, this.keyBindLeft, this.keyBindBack, this.keyBindRight, this.keyBindJump, this.keyBindDrop, this.keyBindInventory, this.keyBindChat, this.keyBindToggleFog, this.keyBindSave, this.keyBindLoad };
        this.numberOfOptions = 9;
        this.difficulty = 2;
        this.thirdPersonView = false;
        this.mc = minecraft;
        this.optionsFile = new File(file, "options.txt");
        this.loadOptions();
    }
    
    public final String setKeyBindingString(final int keyBindingNumber) {
        return this.keyBindings[keyBindingNumber].keyDescription + ": " + Keyboard.getKeyName(this.keyBindings[keyBindingNumber].keyCode);
    }
    
    public final void setKeyBinding(final int keyBindingNumber, final int keyCode) {
        this.keyBindings[keyBindingNumber].keyCode = keyCode;
        this.saveOptions();
    }
    
    public final void setOptionValue(final int optionMenuValue, final int optionSubValue) {
        if (optionMenuValue == 0) {
            this.music = !this.music;
            this.mc.sndManager.onSoundOptionsChanged();
        }
        if (optionMenuValue == 1) {
            this.sound = !this.sound;
            this.mc.sndManager.onSoundOptionsChanged();
        }
        if (optionMenuValue == 2) {
            this.invertMouse = !this.invertMouse;
        }
        if (optionMenuValue == 3) {
            this.showFPS = !this.showFPS;
        }
        if (optionMenuValue == 4) {
            this.renderDistance = (this.renderDistance + optionSubValue & 0x3);
            this.mc.renderGlobal.reload();
        }
        if (optionMenuValue == 5) {
            this.fancyGraphics = !this.fancyGraphics;
        }
        if (optionMenuValue == 6) {
            this.anaglyph = !this.anaglyph;
            this.mc.renderEngine.refreshTextures();
        }
        if (optionMenuValue == 7) {
            this.limitFramerate = !this.limitFramerate;
        }
        if (optionMenuValue == 8) {
            this.difficulty = (this.difficulty + optionSubValue & 0x3);
        }
        this.saveOptions();
    }
    
    public final String setOptionString(final int optionMenuValue) {
        if (optionMenuValue == 0) {
            return new StringBuilder().append("Music: ").append(this.music ? "ON" : "OFF").toString();
        }
        if (optionMenuValue == 1) {
            return new StringBuilder().append("Sound: ").append(this.sound ? "ON" : "OFF").toString();
        }
        if (optionMenuValue == 2) {
            return new StringBuilder().append("Invert mouse: ").append(this.invertMouse ? "ON" : "OFF").toString();
        }
        if (optionMenuValue == 3) {
            return new StringBuilder().append("Show FPS: ").append(this.showFPS ? "ON" : "OFF").toString();
        }
        if (optionMenuValue == 4) {
            return "Render distance: " + GameSettings.RENDER_DISTANCES[this.renderDistance];
        }
        if (optionMenuValue == 5) {
            return new StringBuilder().append("View bobbing: ").append(this.fancyGraphics ? "ON" : "OFF").toString();
        }
        if (optionMenuValue == 6) {
            return new StringBuilder().append("3d anaglyph: ").append(this.anaglyph ? "ON" : "OFF").toString();
        }
        if (optionMenuValue == 7) {
            return new StringBuilder().append("Limit framerate: ").append(this.limitFramerate ? "ON" : "OFF").toString();
        }
        if (optionMenuValue == 8) {
            return "Difficulty: " + GameSettings.DIFFICULTIES[this.difficulty];
        }
        return "";
    }
    
    private void loadOptions() {
        try {
            if (!this.optionsFile.exists()) {
                return;
            }
            final BufferedReader bufferedReader = new BufferedReader((Reader)new FileReader(this.optionsFile));
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                final String[] split;
                if ((split = line.split(":"))[0].equals("music")) {
                    this.music = split[1].equals("true");
                }
                if (split[0].equals("sound")) {
                    this.sound = split[1].equals("true");
                }
                if (split[0].equals("invertYMouse")) {
                    this.invertMouse = split[1].equals("true");
                }
                if (split[0].equals("showFrameRate")) {
                    this.showFPS = split[1].equals("true");
                }
                if (split[0].equals("viewDistance")) {
                    this.renderDistance = Integer.parseInt(split[1]);
                }
                if (split[0].equals("bobView")) {
                    this.fancyGraphics = split[1].equals("true");
                }
                if (split[0].equals("anaglyph3d")) {
                    this.anaglyph = split[1].equals("true");
                }
                if (split[0].equals("limitFramerate")) {
                    this.limitFramerate = split[1].equals("true");
                }
                if (split[0].equals("difficulty")) {
                    this.difficulty = Integer.parseInt(split[1]);
                }
                for (int i = 0; i < this.keyBindings.length; ++i) {
                    if (split[0].equals(("key_" + this.keyBindings[i].keyDescription))) {
                        this.keyBindings[i].keyCode = Integer.parseInt(split[1]);
                    }
                }
            }
            bufferedReader.close();
        }
        catch (Exception ex) {
            System.out.println("Failed to load options");
            ex.printStackTrace();
        }
    }
    
    public final void saveOptions() {
        try {
            final PrintWriter printWriter;
            (printWriter = new PrintWriter((Writer)new FileWriter(this.optionsFile))).println(new StringBuilder().append("music:").append(this.music).toString());
            printWriter.println(new StringBuilder().append("sound:").append(this.sound).toString());
            printWriter.println(new StringBuilder().append("invertYMouse:").append(this.invertMouse).toString());
            printWriter.println(new StringBuilder().append("showFrameRate:").append(this.showFPS).toString());
            printWriter.println(new StringBuilder().append("viewDistance:").append(this.renderDistance).toString());
            printWriter.println(new StringBuilder().append("bobView:").append(this.fancyGraphics).toString());
            printWriter.println(new StringBuilder().append("anaglyph3d:").append(this.anaglyph).toString());
            printWriter.println(new StringBuilder().append("limitFramerate:").append(this.limitFramerate).toString());
            printWriter.println(new StringBuilder().append("difficulty:").append(this.difficulty).toString());
            for (int i = 0; i < this.keyBindings.length; ++i) {
                printWriter.println("key_" + this.keyBindings[i].keyDescription + ":" + this.keyBindings[i].keyCode);
            }
            printWriter.close();
        }
        catch (Exception ex) {
            System.out.println("Failed to save options");
            ex.printStackTrace();
        }
    }
    
    static {
        RENDER_DISTANCES = new String[] { "FAR", "NORMAL", "SHORT", "TINY" };
        DIFFICULTIES = new String[] { "Peaceful", "Easy", "Normal", "Hard" };
    }
}
