package net.minecraft.client.gui;

import java.awt.image.BufferedImage;
import net.minecraft.client.renderer.Tessellator;
import org.lwjgl.opengl.GL11;
import java.io.IOException;
import javax.imageio.ImageIO;
import org.lwjgl.BufferUtils;
import net.minecraft.client.renderer.RenderEngine;
import net.minecraft.client.GameSettings;
import java.nio.IntBuffer;

public final class FontRenderer {
    private int[] charWidth;
    private int fontTextureName;
    private int fontDisplayLists;
    private IntBuffer buffer;
    
    public FontRenderer(final GameSettings gameSettings, final String string, final RenderEngine renderEngine) {
        this.charWidth = new int[256];
        this.fontTextureName = 0;
        this.buffer = BufferUtils.createIntBuffer(1024);
        BufferedImage read;
        try {
            read = ImageIO.read(RenderEngine.class.getResourceAsStream(string));
        }
        catch (IOException ex) {
            throw new RuntimeException((Throwable)ex);
        }
        final int width = read.getWidth();
        int i = read.getHeight();
        final int[] array = new int[width * i];
        read.getRGB(0, 0, width, i, array, 0, width);
        for (int j = 0; j < 128; ++j) {
            i = j % 16;
            final int n = j / 16;
            int n2 = 0;
            for (int n3 = 0; n2 < 8 && n3 == 0; ++n2) {
                final int n4 = (i << 3) + n2;
                n3 = 1;
                for (int n5 = 0; n5 < 8 && n3 != 0; ++n5) {
                    final int n6 = ((n << 3) + n5) * width;
                    final int n7;
                    if ((n7 = (array[n4 + n6] & 0xFF)) > 128) {
                        n3 = 0;
                    }
                }
            }
            if (j == 32) {
                n2 = 4;
            }
            this.charWidth[j] = n2;
        }
        this.fontTextureName = renderEngine.getTexture(string);
        this.fontDisplayLists = GL11.glGenLists(288);
        final Tessellator instance = Tessellator.instance;
        for (i = 0; i < 256; ++i) {
            GL11.glNewList(this.fontDisplayLists + i, 4864);
            instance.startDrawingQuads();
            final int n = i % 16 << 3;
            final int n2 = i / 16 << 3;
            instance.addVertexWithUV(0.0f, 7.99f, 0.0f, n / 128.0f, (n2 + 7.99f) / 128.0f);
            instance.addVertexWithUV(7.99f, 7.99f, 0.0f, (n + 7.99f) / 128.0f, (n2 + 7.99f) / 128.0f);
            instance.addVertexWithUV(7.99f, 0.0f, 0.0f, (n + 7.99f) / 128.0f, n2 / 128.0f);
            instance.addVertexWithUV(0.0f, 0.0f, 0.0f, n / 128.0f, n2 / 128.0f);
            instance.draw();
            GL11.glTranslated((double)this.charWidth[i], 0.0f, 0.0f);
            GL11.glEndList();
        }
        for (i = 0; i < 32; ++i) {
            final int n = (i & 0x8) << 3;
            int n2 = (i & 0x1) * 191 + n;
            int n8 = ((i & 0x2) >> 1) * 191 + n;
            int n4 = ((i & 0x4) >> 2) * 191 + n;
            final boolean b = i >= 16;
            if (gameSettings.anaglyph) {
                final int n6 = (n4 * 30 + n8 * 59 + n2 * 11) / 100;
                final int n7 = (n4 * 30 + n8 * 70) / 100;
                final int n9 = (n4 * 30 + n2 * 70) / 100;
                n4 = n6;
                n8 = n7;
                n2 = n9;
            }
            i += 2;
            if (b) {
                n4 /= 4;
                n8 /= 4;
                n2 /= 4;
            }
            GL11.glColor4d(n4 / 255.0f, n8 / 255.0f, n2 / 255.0f, 1.0f);
        }
    }
    
    public final void drawStringWithShadow(final String string, final int xPosition, final int yPosition, final int fontColor) {
        this.renderString(string, xPosition + 1, yPosition + 1, fontColor, true);
        this.drawString(string, xPosition, yPosition, fontColor);
    }
    
    public final void drawString(final String string, final int xPosition, final int yPosition, final int fontColor) {
        this.renderString(string, xPosition, yPosition, fontColor, false);
    }
    
    private void renderString(final String string, final int xPosition, final int yPosition, int fontColor, final boolean textShadow) {
        if (string == null) {
            return;
        }
        final char[] charArray = string.toCharArray();
        if (textShadow) {
            fontColor = (fontColor & 0xFCFCFC) >> 2;
        }
        GL11.glBindTexture(3553, this.fontTextureName);
        GL11.glColor4d((fontColor >> 16 & 0xFF) / 255.0f, (fontColor >> 8 & 0xFF) / 255.0f, (fontColor & 0xFF) / 255.0f, 1.0f);
        this.buffer.clear();
        GL11.glPushMatrix();
        GL11.glTranslated((double)xPosition, (double)yPosition, 0.0f);
        for (int i = 0; i < charArray.length; ++i) {
            while (charArray[i] == '&' && charArray.length > i + 1) {
                int index;
                if ((index = "0123456789abcdef".indexOf((int)charArray[i + 1])) < 0 || index > 15) {
                    index = 15;
                }
                this.buffer.put(this.fontDisplayLists + 256 + index + (textShadow ? 16 : 0));
                if (this.buffer.remaining() == 0) {
                    this.buffer.flip();
                    GL11.glCallLists(this.buffer);
                    this.buffer.clear();
                }
                i += 2;
            }
            this.buffer.put(this.fontDisplayLists + charArray[i]);
            if (this.buffer.remaining() == 0) {
                this.buffer.flip();
                GL11.glCallLists(this.buffer);
                this.buffer.clear();
            }
        }
        this.buffer.flip();
        GL11.glCallLists(this.buffer);
        GL11.glPopMatrix();
    }
    
    public final int getStringWidth(final String string) {
        if (string == null) {
            return 0;
        }
        final char[] charArray = string.toCharArray();
        int n = 0;
        for (int i = 0; i < charArray.length; ++i) {
            if (charArray[i] == '&') {
                ++i;
            }
            else {
                n += this.charWidth[charArray[i]];
            }
        }
        return n;
    }
}
