package net.minecraft.client.model;

import net.minecraft.game.MathHelper;

public class ModelBiped extends ModelBase {
    public ModelRenderer bipedHead;
    public ModelRenderer bipedHeadwear;
    public ModelRenderer bipedBody;
    public ModelRenderer bipedRightArm;
    public ModelRenderer bipedLeftArm;
    public ModelRenderer bipedRightLeg;
    public ModelRenderer bipedLeftLeg;
    
    public ModelBiped() {
        this(0.0f);
    }
    
    public ModelBiped(final double scale) {
        this(scale, 0.0f);
    }
    
    private ModelBiped(final double scale, final double double2) {
        (this.bipedHead = new ModelRenderer(0, 0)).addBox(-4.0f, -8.0f, -4.0f, 8, 8, 8, scale);
        this.bipedHead.setRotationPoint(0.0f, 0.0f, 0.0f);
        (this.bipedHeadwear = new ModelRenderer(32, 0)).addBox(-4.0f, -8.0f, -4.0f, 8, 8, 8, scale + 0.5f);
        this.bipedHeadwear.setRotationPoint(0.0f, 0.0f, 0.0f);
        (this.bipedBody = new ModelRenderer(16, 16)).addBox(-4.0f, 0.0f, -2.0f, 8, 12, 4, scale);
        this.bipedBody.setRotationPoint(0.0f, 0.0f, 0.0f);
        (this.bipedRightArm = new ModelRenderer(40, 16)).addBox(-3.0f, -2.0f, -2.0f, 4, 12, 4, scale);
        this.bipedRightArm.setRotationPoint(-5.0f, 2.0f, 0.0f);
        this.bipedLeftArm = new ModelRenderer(40, 16);
        this.bipedLeftArm.mirror = true;
        this.bipedLeftArm.addBox(-1.0f, -2.0f, -2.0f, 4, 12, 4, scale);
        this.bipedLeftArm.setRotationPoint(5.0f, 2.0f, 0.0f);
        (this.bipedRightLeg = new ModelRenderer(0, 16)).addBox(-2.0f, 0.0f, -2.0f, 4, 12, 4, scale);
        this.bipedRightLeg.setRotationPoint(-2.0f, 12.0f, 0.0f);
        this.bipedLeftLeg = new ModelRenderer(0, 16);
        this.bipedLeftLeg.mirror = true;
        this.bipedLeftLeg.addBox(-2.0f, 0.0f, -2.0f, 4, 12, 4, scale);
        this.bipedLeftLeg.setRotationPoint(2.0f, 12.0f, 0.0f);
    }
    
    @Override
    public final void render(final double limbAngle, final double animationSpeed, final double double3, final double headYaw, final double headPitch, final double double6) {
        this.setRotationAngles(limbAngle, animationSpeed, double3, headYaw, headPitch, 1.0f);
        this.bipedHead.render(1.0f);
        this.bipedBody.render(1.0f);
        this.bipedRightArm.render(1.0f);
        this.bipedLeftArm.render(1.0f);
        this.bipedRightLeg.render(1.0f);
        this.bipedLeftLeg.render(1.0f);
        this.bipedHeadwear.render(1.0f);
    }
    
    @Override
    public void setRotationAngles(final double limbAngle, final double animationSpeed, final double double3, final double headYaw, final double headPitch, final double double6) {
        this.bipedHead.rotateAngleY = headYaw / 57.295776f;
        this.bipedHead.rotateAngleX = headPitch / 57.295776f;
        this.bipedHeadwear.rotateAngleY = this.bipedHead.rotateAngleY;
        this.bipedHeadwear.rotateAngleX = this.bipedHead.rotateAngleX;
        this.bipedRightArm.rotateAngleX = MathHelper.cos(limbAngle * 0.6662f + 3.1415927f) * 2.0f * animationSpeed;
        this.bipedRightArm.rotateAngleZ = (MathHelper.cos(limbAngle * 0.2312f) + 1.0f) * animationSpeed;
        this.bipedLeftArm.rotateAngleX = MathHelper.cos(limbAngle * 0.6662f) * 2.0f * animationSpeed;
        this.bipedLeftArm.rotateAngleZ = (MathHelper.cos(limbAngle * 0.2812f) - 1.0f) * animationSpeed;
        this.bipedRightLeg.rotateAngleX = MathHelper.cos(limbAngle * 0.6662f) * 1.4f * animationSpeed;
        this.bipedLeftLeg.rotateAngleX = MathHelper.cos(limbAngle * 0.6662f + 3.1415927f) * 1.4f * animationSpeed;
        final ModelRenderer bipedRightArm = this.bipedRightArm;
        bipedRightArm.rotateAngleZ += MathHelper.cos(double3 * 0.09f) * 0.05f + 0.05f;
        final ModelRenderer bipedLeftArm = this.bipedLeftArm;
        bipedLeftArm.rotateAngleZ -= MathHelper.cos(double3 * 0.09f) * 0.05f + 0.05f;
        final ModelRenderer bipedRightArm2 = this.bipedRightArm;
        bipedRightArm2.rotateAngleX += MathHelper.sin(double3 * 0.067f) * 0.05f;
        final ModelRenderer bipedLeftArm2 = this.bipedLeftArm;
        bipedLeftArm2.rotateAngleX -= MathHelper.sin(double3 * 0.067f) * 0.05f;
    }
}
