package net.minecraft.client.model;

import net.minecraft.client.renderer.Vec3D;
import net.minecraft.client.renderer.Tessellator;
import org.lwjgl.opengl.GL11;

public final class ModelRenderer {
    private PositionTextureVertex[] corners;
    private TexturedQuad[] faces;
    private int textureOffsetX;
    private int textureOffsetY;
    private double rotationPointX;
    private double rotationPointY;
    private double rotationPointZ;
    public double rotateAngleX;
    public double rotateAngleY;
    public double rotateAngleZ;
    private boolean compiled;
    private int displayList;
    public boolean mirror;
    public boolean showModel;
    private boolean isHidden;
    
    public ModelRenderer(final int integer1, final int integer2) {
        this.compiled = false;
        this.displayList = 0;
        this.mirror = false;
        this.showModel = true;
        this.isHidden = false;
        this.textureOffsetX = integer1;
        this.textureOffsetY = integer2;
    }
    
    public final void addBox(double double1, double double2, double double3, int integer4, final int integer5, final int integer6, double double7) {
        this.corners = new PositionTextureVertex[8];
        this.faces = new TexturedQuad[6];
        double n = double1 + integer4;
        double n2 = double2 + integer5;
        double n3 = double3 + integer6;
        double1 -= double7;
        double2 -= double7;
        double3 -= double7;
        n += double7;
        n2 += double7;
        n3 += double7;
        if (this.mirror) {
            double7 = n;
            n = double1;
            double1 = double7;
        }
        final PositionTextureVertex positionTextureVertex = new PositionTextureVertex(double1, double2, double3, 0.0f, 0.0f);
        final PositionTextureVertex positionTextureVertex2 = new PositionTextureVertex(n, double2, double3, 0.0f, 8.0f);
        final PositionTextureVertex positionTextureVertex3 = new PositionTextureVertex(n, n2, double3, 8.0f, 8.0f);
        final PositionTextureVertex positionTextureVertex4 = new PositionTextureVertex(double1, n2, double3, 8.0f, 0.0f);
        final PositionTextureVertex positionTextureVertex5 = new PositionTextureVertex(double1, double2, n3, 0.0f, 0.0f);
        final PositionTextureVertex positionTextureVertex6 = new PositionTextureVertex(n, double2, n3, 0.0f, 8.0f);
        final PositionTextureVertex positionTextureVertex7 = new PositionTextureVertex(n, n2, n3, 8.0f, 8.0f);
        final PositionTextureVertex positionTextureVertex8 = new PositionTextureVertex(double1, n2, n3, 8.0f, 0.0f);
        this.corners[0] = positionTextureVertex;
        this.corners[1] = positionTextureVertex2;
        this.corners[2] = positionTextureVertex3;
        this.corners[3] = positionTextureVertex4;
        this.corners[4] = positionTextureVertex5;
        this.corners[5] = positionTextureVertex6;
        this.corners[6] = positionTextureVertex7;
        this.corners[7] = positionTextureVertex8;
        this.faces[0] = new TexturedQuad(new PositionTextureVertex[] { positionTextureVertex6, positionTextureVertex2, positionTextureVertex3, positionTextureVertex7 }, this.textureOffsetX + integer6 + integer4, this.textureOffsetY + integer6, this.textureOffsetX + integer6 + integer4 + integer6, this.textureOffsetY + integer6 + integer5);
        this.faces[1] = new TexturedQuad(new PositionTextureVertex[] { positionTextureVertex, positionTextureVertex5, positionTextureVertex8, positionTextureVertex4 }, this.textureOffsetX, this.textureOffsetY + integer6, this.textureOffsetX + integer6, this.textureOffsetY + integer6 + integer5);
        this.faces[2] = new TexturedQuad(new PositionTextureVertex[] { positionTextureVertex6, positionTextureVertex5, positionTextureVertex, positionTextureVertex2 }, this.textureOffsetX + integer6, this.textureOffsetY, this.textureOffsetX + integer6 + integer4, this.textureOffsetY + integer6);
        this.faces[3] = new TexturedQuad(new PositionTextureVertex[] { positionTextureVertex3, positionTextureVertex4, positionTextureVertex8, positionTextureVertex7 }, this.textureOffsetX + integer6 + integer4, this.textureOffsetY, this.textureOffsetX + integer6 + integer4 + integer4, this.textureOffsetY + integer6);
        this.faces[4] = new TexturedQuad(new PositionTextureVertex[] { positionTextureVertex2, positionTextureVertex, positionTextureVertex4, positionTextureVertex3 }, this.textureOffsetX + integer6, this.textureOffsetY + integer6, this.textureOffsetX + integer6 + integer4, this.textureOffsetY + integer6 + integer5);
        this.faces[5] = new TexturedQuad(new PositionTextureVertex[] { positionTextureVertex5, positionTextureVertex6, positionTextureVertex7, positionTextureVertex8 }, this.textureOffsetX + integer6 + integer4 + integer6, this.textureOffsetY + integer6, this.textureOffsetX + integer6 + integer4 + integer6 + integer4, this.textureOffsetY + integer6 + integer5);
        if (this.mirror) {
            for (int i = 0; i < this.faces.length; ++i) {
                TexturedQuad texturedQuad;
                PositionTextureVertex[] vertexPositions;
                for (vertexPositions = new PositionTextureVertex[(texturedQuad = this.faces[i]).vertexPositions.length], integer4 = 0; integer4 < texturedQuad.vertexPositions.length; ++integer4) {
                    vertexPositions[integer4] = texturedQuad.vertexPositions[texturedQuad.vertexPositions.length - integer4 - 1];
                }
                texturedQuad.vertexPositions = vertexPositions;
            }
        }
    }
    
    public final void setRotationPoint(final double double1, final double double2, final double double3) {
        this.rotationPointX = double1;
        this.rotationPointY = double2;
        this.rotationPointZ = double3;
    }
    
    public final void render(final double double1) {
        if (!this.showModel) {
            return;
        }
        if (!this.compiled) {
            GL11.glNewList(this.displayList = GL11.glGenLists(1), 4864);
            final Tessellator instance = Tessellator.instance;
            for (int i = 0; i < this.faces.length; ++i) {
                instance.startDrawingQuads();
                final TexturedQuad texturedQuad = this.faces[i];
                final Tessellator tessellator = instance;
                final TexturedQuad texturedQuad2 = texturedQuad;
                final Vec3D normalize = texturedQuad.vertexPositions[1].vector3D.subtract(texturedQuad2.vertexPositions[0].vector3D).normalize();
                final Vec3D normalize2 = texturedQuad2.vertexPositions[1].vector3D.subtract(texturedQuad2.vertexPositions[2].vector3D).normalize();
                final Vec3D vec3D = normalize;
                final Vec3D vec3D2 = normalize2;
                final Vec3D vec3D3 = vec3D;
                final Vec3D normalize3;
                Tessellator.setNormal(-(normalize3 = new Vec3D(vec3D3.yCoord * vec3D2.zCoord - vec3D3.zCoord * vec3D2.yCoord, vec3D3.zCoord * vec3D2.xCoord - vec3D3.xCoord * vec3D2.zCoord, vec3D3.xCoord * vec3D2.yCoord - vec3D3.yCoord * vec3D2.xCoord).normalize()).xCoord, -normalize3.yCoord, -normalize3.zCoord);
                for (int j = 0; j < 4; ++j) {
                    final PositionTextureVertex positionTextureVertex = texturedQuad2.vertexPositions[j];
                    tessellator.addVertexWithUV(positionTextureVertex.vector3D.xCoord * double1, positionTextureVertex.vector3D.yCoord * double1, positionTextureVertex.vector3D.zCoord * double1, positionTextureVertex.texturePositionX, positionTextureVertex.texturePositionY);
                }
                instance.draw();
            }
            GL11.glEndList();
            this.compiled = true;
        }
        if (this.rotateAngleX != 0.0f || this.rotateAngleY != 0.0f || this.rotateAngleZ != 0.0f) {
            GL11.glPushMatrix();
            GL11.glTranslated(this.rotationPointX * double1, this.rotationPointY * double1, this.rotationPointZ * double1);
            if (this.rotateAngleZ != 0.0f) {
                GL11.glRotated(this.rotateAngleZ * 57.295776f, 0.0f, 0.0f, 1.0f);
            }
            if (this.rotateAngleY != 0.0f) {
                GL11.glRotated(this.rotateAngleY * 57.295776f, 0.0f, 1.0f, 0.0f);
            }
            if (this.rotateAngleX != 0.0f) {
                GL11.glRotated(this.rotateAngleX * 57.295776f, 1.0f, 0.0f, 0.0f);
            }
            GL11.glCallList(this.displayList);
            GL11.glPopMatrix();
            return;
        }
        if (this.rotationPointX != 0.0f || this.rotationPointY != 0.0f || this.rotationPointZ != 0.0f) {
            GL11.glTranslated(this.rotationPointX * double1, this.rotationPointY * double1, this.rotationPointZ * double1);
            GL11.glCallList(this.displayList);
            GL11.glTranslated(-this.rotationPointX * double1, -this.rotationPointY * double1, -this.rotationPointZ * double1);
            return;
        }
        GL11.glCallList(this.displayList);
    }
}
