package net.minecraft.client.player;

import net.minecraft.game.level.World;
import net.minecraft.game.entity.player.EntityPlayer;
import net.minecraft.game.item.ItemStack;
import net.minecraft.game.block.Block;
import net.minecraft.game.item.Item;
import net.minecraft.client.Minecraft;
import net.minecraft.game.level.MobSpawner;

public final class PlayerControllerSurvival extends PlayerController {
    private int curBlockX;
    private int curBlockY;
    private int curBlockZ;
    private double curBlockDamage;
    private double prevBlockDamage;
    private double blockDestroySoundCounter;
    private int blockHitWait;
    private MobSpawner mobSpawner;
    
    public PlayerControllerSurvival(final Minecraft minecraft) {
        super(minecraft);
        this.curBlockX = -1;
        this.curBlockY = -1;
        this.curBlockZ = -1;
        this.curBlockDamage = 0.0f;
        this.prevBlockDamage = 0.0f;
        this.blockDestroySoundCounter = 0.0f;
        this.blockHitWait = 0;
    }
    
    @Override
    public final boolean sendBlockRemoved(final int integer1, final int integer2, final int integer3) {
        final int blockId = this.mc.theWorld.getBlockId(integer1, integer2, integer3);
        final byte blockMetadata = this.mc.theWorld.getBlockMetadata(integer1, integer2, integer3);
        final boolean sendBlockRemoved = super.sendBlockRemoved(integer1, integer2, integer3);
        final ItemStack currentItem;
        if ((currentItem = this.mc.thePlayer.inventory.getCurrentItem()) != null) {
            final ItemStack itemStack = currentItem;
            Item.itemsList[itemStack.itemID].onBlockDestroyed(itemStack);
            if (currentItem.stackSize == 0) {
                this.mc.thePlayer.displayGUIInventory();
            }
        }
        if (sendBlockRemoved && this.mc.thePlayer.canHarvestBlock(Block.blocksList[blockId])) {
            Block.blocksList[blockId].dropBlockAsItem(this.mc.theWorld, integer1, integer2, integer3, blockMetadata);
        }
        return sendBlockRemoved;
    }
    
    @Override
    public final void clickBlock(final int integer1, final int integer2, final int integer3) {
        final int blockId;
        if ((blockId = this.mc.theWorld.getBlockId(integer1, integer2, integer3)) > 0 && Block.blocksList[blockId].blockStrength(this.mc.thePlayer) >= 1.0f) {
            this.sendBlockRemoved(integer1, integer2, integer3);
        }
    }
    
    @Override
    public final void resetBlockRemoving() {
        this.curBlockDamage = 0.0f;
        this.blockHitWait = 0;
    }
    
    @Override
    public final void sendBlockRemoving(final int integer1, final int integer2, final int integer3, int integer4) {
        if (this.blockHitWait > 0) {
            --this.blockHitWait;
            return;
        }
        super.sendBlockRemoving(integer1, integer2, integer3, integer4);
        if (integer1 != this.curBlockX || integer2 != this.curBlockY || integer3 != this.curBlockZ) {
            this.curBlockDamage = 0.0f;
            this.prevBlockDamage = 0.0f;
            this.blockDestroySoundCounter = 0.0f;
            this.curBlockX = integer1;
            this.curBlockY = integer2;
            this.curBlockZ = integer3;
            return;
        }
        if ((integer4 = this.mc.theWorld.getBlockId(integer1, integer2, integer3)) == 0) {
            return;
        }
        final Block block = Block.blocksList[integer4];
        this.curBlockDamage += block.blockStrength(this.mc.thePlayer);
        if (this.blockDestroySoundCounter % 4.0f == 0.0f && block != null) {
            this.mc.sndManager.playSound(block.stepSound.stepSoundDir2(), integer1 + 0.5f, integer2 + 0.5f, integer3 + 0.5f, (block.stepSound.soundVolume + 1.0f) / 8.0f, block.stepSound.soundPitch * 0.5f);
        }
        ++this.blockDestroySoundCounter;
        if (this.curBlockDamage >= 1.0f) {
            this.sendBlockRemoved(integer1, integer2, integer3);
            this.curBlockDamage = 0.0f;
            this.prevBlockDamage = 0.0f;
            this.blockDestroySoundCounter = 0.0f;
            this.blockHitWait = 5;
        }
    }
    
    @Override
    public final void setPartialTime(double double1) {
        if (this.curBlockDamage <= 0.0f) {
            this.mc.renderGlobal.damagePartialTime = 0.0f;
            return;
        }
        double1 = this.prevBlockDamage + (this.curBlockDamage - this.prevBlockDamage) * double1;
        this.mc.renderGlobal.damagePartialTime = double1;
    }
    
    @Override
    public final double getBlockReachDistance() {
        return 4.0f;
    }
    
    @Override
    public final void onWorldChange(final World world) {
        super.onWorldChange(world);
        this.mobSpawner = new MobSpawner(world);
    }
    
    @Override
    public final void updateController() {
        this.prevBlockDamage = this.curBlockDamage;
        this.mobSpawner.spawn();
    }
}
