package net.minecraft.client.renderer;

import java.util.Iterator;
import net.minecraft.client.renderer.block.TextureFX;
import java.awt.Graphics;
import java.awt.image.ImageObserver;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.imageio.ImageIO;
import org.lwjgl.opengl.GL11;
import java.util.ArrayList;
import org.lwjgl.BufferUtils;
import net.minecraft.client.GameSettings;
import java.util.Map;
import java.util.List;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.util.HashMap;

public class RenderEngine {
    private HashMap<String,Object> textureMap;
    private HashMap<Integer,Object> textureNameToImageMap;
    private IntBuffer singleIntBuffer;
    private ByteBuffer imageData;
    private List textureList;
    private Map urlToImageDataMap;
    private GameSettings options;
    private boolean clampTexture;
    
    public RenderEngine(final GameSettings gameSettings) {
        this.textureMap = new HashMap();
        this.textureNameToImageMap = new HashMap();
        this.singleIntBuffer = BufferUtils.createIntBuffer(1);
        this.imageData = BufferUtils.createByteBuffer(262144);
        this.textureList = (List)new ArrayList();
        this.urlToImageDataMap = (Map)new HashMap();
        this.clampTexture = false;
        this.options = gameSettings;
    }
    
    public final int getTexture(final String string) {
        final Integer n;
        if ((n = (Integer)this.textureMap.get(string)) != null) {
            return n;
        }
        try {
            this.singleIntBuffer.clear();
            GL11.glGenTextures(this.singleIntBuffer);
            final int value = this.singleIntBuffer.get(0);
            if (string.startsWith("##")) {
                this.setupTexture(unwrapImageByColumns(ImageIO.read(RenderEngine.class.getResourceAsStream(string.substring(2)))), value);
            }
            else if (string.startsWith("%%")) {
                this.clampTexture = true;
                this.setupTexture(ImageIO.read(RenderEngine.class.getResourceAsStream(string.substring(2))), value);
                this.clampTexture = false;
            }
            else {
                this.setupTexture(ImageIO.read(RenderEngine.class.getResourceAsStream(string)), value);
            }
            this.textureMap.put(string, value);
            return value;
        }
        catch (IOException ex) {
            throw new RuntimeException("!!");
        }
    }
    
    private static BufferedImage unwrapImageByColumns(final BufferedImage bufferedImage) {
        final int n = bufferedImage.getWidth() / 16;
        final BufferedImage bufferedImage2;
        final Graphics graphics = (bufferedImage2 = new BufferedImage(16, bufferedImage.getHeight() * n, 2)).getGraphics();
        for (int i = 0; i < n; ++i) {
            graphics.drawImage((Image)bufferedImage, -i << 4, i * bufferedImage.getHeight(), (ImageObserver)null);
        }
        graphics.dispose();
        return bufferedImage2;
    }
    
    private void setupTexture(final BufferedImage bufferedImage, int integer) {
        GL11.glBindTexture(3553, integer);
        GL11.glTexParameteri(3553, 10241, 9728);
        GL11.glTexParameteri(3553, 10240, 9728);
        if (this.clampTexture) {
            GL11.glTexParameteri(3553, 10242, 10496);
            GL11.glTexParameteri(3553, 10243, 10496);
        }
        else {
            GL11.glTexParameteri(3553, 10242, 10497);
            GL11.glTexParameteri(3553, 10243, 10497);
        }
        integer = bufferedImage.getWidth();
        final int height = bufferedImage.getHeight();
        final int[] array = new int[integer * height];
        final byte[] array2 = new byte[integer * height << 2];
        bufferedImage.getRGB(0, 0, integer, height, array, 0, integer);
        for (int i = 0; i < array.length; ++i) {
            final int n = array[i] >>> 24;
            int n2 = array[i] >> 16 & 0xFF;
            int n3 = array[i] >> 8 & 0xFF;
            int n4 = array[i] & 0xFF;
            if (this.options != null && this.options.anaglyph) {
                final int n5 = (n2 * 30 + n3 * 59 + n4 * 11) / 100;
                n3 = (n2 * 30 + n3 * 70) / 100;
                n4 = (n2 * 30 + n4 * 70) / 100;
                n2 = n5;
                n3 = n3;
                n4 = n4;
            }
            array2[i << 2] = (byte)n2;
            array2[(i << 2) + 1] = (byte)n3;
            array2[(i << 2) + 2] = (byte)n4;
            array2[(i << 2) + 3] = (byte)n;
        }
        this.imageData.clear();
        this.imageData.put(array2);
        this.imageData.position(0).limit(array2.length);
        GL11.glTexImage2D(3553, 0, 6408, integer, height, 0, 6408, 5121, this.imageData);
    }
    
    public final int getTextureForDownloadableImage(final String string1, final String string2) {
        final ThreadDownloadImageData threadDownloadImageData;
        if ((threadDownloadImageData = (ThreadDownloadImageData)this.urlToImageDataMap.get(string1)) != null && threadDownloadImageData.image != null && !threadDownloadImageData.textureSetupComplete) {
            if (threadDownloadImageData.textureName < 0) {
                final ThreadDownloadImageData threadDownloadImageData2 = threadDownloadImageData;
                final BufferedImage image = threadDownloadImageData.image;
                this.singleIntBuffer.clear();
                GL11.glGenTextures(this.singleIntBuffer);
                final int value = this.singleIntBuffer.get(0);
                this.setupTexture(image, value);
                this.textureNameToImageMap.put(value, image);
                threadDownloadImageData2.textureName = value;
            }
            else {
                this.setupTexture(threadDownloadImageData.image, threadDownloadImageData.textureName);
            }
            threadDownloadImageData.textureSetupComplete = true;
        }
        if (threadDownloadImageData == null || threadDownloadImageData.textureName < 0) {
            return this.getTexture(string2);
        }
        return threadDownloadImageData.textureName;
    }
    
    public final ThreadDownloadImageData obtainImageData(final String string, final ImageBufferDownload imageBufferDownload) {
        final ThreadDownloadImageData threadDownloadImageData;
        if ((threadDownloadImageData = (ThreadDownloadImageData)this.urlToImageDataMap.get(string)) == null) {
            this.urlToImageDataMap.put(string, new ThreadDownloadImageData(string, imageBufferDownload));
        }
        else {
            final ThreadDownloadImageData threadDownloadImageData2 = threadDownloadImageData;
            ++threadDownloadImageData2.referenceCount;
        }
        return threadDownloadImageData;
    }
    
    public final void releaseImageData(final String string) {
        final ThreadDownloadImageData threadDownloadImageData;
        if ((threadDownloadImageData = (ThreadDownloadImageData)this.urlToImageDataMap.get(string)) != null) {
            final ThreadDownloadImageData threadDownloadImageData2 = threadDownloadImageData;
            --threadDownloadImageData2.referenceCount;
            if (threadDownloadImageData.referenceCount == 0) {
                if (threadDownloadImageData.textureName >= 0) {
                    final int textureName = threadDownloadImageData.textureName;
                    this.textureNameToImageMap.remove(textureName);
                    this.singleIntBuffer.clear();
                    this.singleIntBuffer.put(textureName);
                    this.singleIntBuffer.flip();
                    GL11.glDeleteTextures(this.singleIntBuffer);
                }
                this.urlToImageDataMap.remove(string);
            }
        }
    }
    
    public final void registerTextureFX(final TextureFX textureFX) {
        this.textureList.add(textureFX);
        textureFX.onTick();
    }
    
    public final void updateDynamicTextures() {
        for (int i = 0; i < this.textureList.size(); ++i) {
            final TextureFX textureFX;
            (textureFX = (TextureFX)this.textureList.get(i)).anaglyphEnabled = this.options.anaglyph;
            textureFX.onTick();
            this.imageData.clear();
            this.imageData.put(textureFX.imageData);
            this.imageData.position(0).limit(textureFX.imageData.length);
            GL11.glTexSubImage2D(3553, 0, textureFX.iconIndex % 16 << 4, textureFX.iconIndex / 16 << 4, 16, 16, 6408, 5121, this.imageData);
        }
        for (int i = 0; i < this.textureList.size(); ++i) {
            final TextureFX textureFX2;
            if ((textureFX2 = (TextureFX)this.textureList.get(i)).textureId > 0) {
                this.imageData.clear();
                this.imageData.put(textureFX2.imageData);
                this.imageData.position(0).limit(textureFX2.imageData.length);
                GL11.glBindTexture(3553, textureFX2.textureId);
                GL11.glTexSubImage2D(3553, 0, 0, 0, 16, 16, 6408, 5121, this.imageData);
            }
        }
    }
    
    public final void refreshTextures() {
        for (final int n : this.textureNameToImageMap.keySet()) {
            this.setupTexture((BufferedImage)this.textureNameToImageMap.get(n), n);
        }
        final Iterator iterator2 = this.urlToImageDataMap.values().iterator();
        while (iterator2.hasNext()) {
            ((ThreadDownloadImageData)iterator2.next()).textureSetupComplete = false;
        }
        for (final String s : this.textureMap.keySet()) {
            try {
                BufferedImage bufferedImage;
                if (s.startsWith("##")) {
                    bufferedImage = unwrapImageByColumns(ImageIO.read(RenderEngine.class.getResourceAsStream(s.substring(2))));
                }
                else if (s.startsWith("%%")) {
                    this.clampTexture = true;
                    bufferedImage = ImageIO.read(RenderEngine.class.getResourceAsStream(s.substring(2)));
                    this.clampTexture = false;
                }
                else {
                    bufferedImage = ImageIO.read(RenderEngine.class.getResourceAsStream(s));
                }
                final int n = (int)this.textureMap.get(s);
                this.setupTexture(bufferedImage, n);
            }
            catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }
    
    public static void bindTexture(final int integer) {
        if (integer < 0) {
            return;
        }
        GL11.glBindTexture(3553, integer);
    }
}
