package net.minecraft.client.renderer;

import net.minecraft.game.MathHelper;

public final class Vec3D {
    public double xCoord;
    public double yCoord;
    public double zCoord;
    
    public Vec3D(final double double1, final double double2, final double double3) {
        this.xCoord = double1;
        this.yCoord = double2;
        this.zCoord = double3;
    }
    
    public final Vec3D subtract(final Vec3D vec3D) {
        return new Vec3D(this.xCoord - vec3D.xCoord, this.yCoord - vec3D.yCoord, this.zCoord - vec3D.zCoord);
    }
    
    public final Vec3D normalize() {
        final double sqrt_double = MathHelper.sqrt_double(this.xCoord * this.xCoord + this.yCoord * this.yCoord + this.zCoord * this.zCoord);
        return new Vec3D(this.xCoord / sqrt_double, this.yCoord / sqrt_double, this.zCoord / sqrt_double);
    }
    
    public final Vec3D addVector(final double double1, final double double2, final double double3) {
        return new Vec3D(this.xCoord + double1, this.yCoord + double2, this.zCoord + double3);
    }
    
    public final double distance(final Vec3D vec3D) {
        final double n = vec3D.xCoord - this.xCoord;
        final double n2 = vec3D.yCoord - this.yCoord;
        final double n3 = vec3D.zCoord - this.zCoord;
        return MathHelper.sqrt_double(n * n + n2 * n2 + n3 * n3);
    }
    
    public final double squareDistanceTo(final Vec3D vec3D) {
        final double n = vec3D.xCoord - this.xCoord;
        final double n2 = vec3D.yCoord - this.yCoord;
        final double n3 = vec3D.zCoord - this.zCoord;
        return n * n + n2 * n2 + n3 * n3;
    }
    
    public final Vec3D getIntermediateWithXValue(final Vec3D vec3D, double double2) {
        final double n = vec3D.xCoord - this.xCoord;
        final double n2 = vec3D.yCoord - this.yCoord;
        final double n3 = vec3D.zCoord - this.zCoord;
        if (n * n < 1.0E-7f) {
            return null;
        }
        if ((double2 = (double2 - this.xCoord) / n) < 0.0f || double2 > 1.0f) {
            return null;
        }
        return new Vec3D(this.xCoord + n * double2, this.yCoord + n2 * double2, this.zCoord + n3 * double2);
    }
    
    public final Vec3D getIntermediateWithYValue(final Vec3D vec3D, double double2) {
        final double n = vec3D.xCoord - this.xCoord;
        final double n2 = vec3D.yCoord - this.yCoord;
        final double n3 = vec3D.zCoord - this.zCoord;
        if (n2 * n2 < 1.0E-7f) {
            return null;
        }
        if ((double2 = (double2 - this.yCoord) / n2) < 0.0f || double2 > 1.0f) {
            return null;
        }
        return new Vec3D(this.xCoord + n * double2, this.yCoord + n2 * double2, this.zCoord + n3 * double2);
    }
    
    public final Vec3D getIntermediateWithZValue(final Vec3D vec3D, double double2) {
        final double n = vec3D.xCoord - this.xCoord;
        final double n2 = vec3D.yCoord - this.yCoord;
        final double n3;
        if ((n3 = vec3D.zCoord - this.zCoord) * n3 < 1.0E-7f) {
            return null;
        }
        if ((double2 = (double2 - this.zCoord) / n3) < 0.0f || double2 > 1.0f) {
            return null;
        }
        return new Vec3D(this.xCoord + n * double2, this.yCoord + n2 * double2, this.zCoord + n3 * double2);
    }
    
    public final String toString() {
        return new StringBuilder().append("(").append(this.xCoord).append(", ").append(this.yCoord).append(", ").append(this.zCoord).append(")").toString();
    }
}
