package net.minecraft.client.renderer.entity;

import net.minecraft.game.level.World;
import net.minecraft.game.block.Block;
import net.minecraft.client.renderer.Tessellator;
import org.lwjgl.opengl.GL11;
import net.minecraft.client.physics.AxisAlignedBB;
import net.minecraft.client.renderer.RenderEngine;
import net.minecraft.game.entity.Entity;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.model.ModelBiped;

public abstract class Render {
    protected RenderManager renderManager;
    protected double shadowSize;
    protected double shadowOpaque;
    
    public Render() {
        new ModelBiped();
        new RenderBlocks();
        this.shadowSize = 0.0f;
        this.shadowOpaque = 1.0f;
    }
    
    public abstract void doRender(final Entity entity, final double double2, final double double3, final double double4, final double double5, final double double6);
    
    protected final void loadTexture(final String string) {
        RenderEngine.bindTexture(this.renderManager.renderEngine.getTexture(string));
    }
    
    protected final void loadDownloadableImageTexture(final String string1, final String string2) {
        RenderEngine.bindTexture(this.renderManager.renderEngine.getTextureForDownloadableImage(string1, string2));
    }
    
    public static void renderOffsetAABB(final AxisAlignedBB aabb) {
        GL11.glDisable(3553);
        final Tessellator instance = Tessellator.instance;
        GL11.glColor4d(1.0f, 1.0f, 1.0f, 1.0f);
        instance.startDrawingQuads();
        Tessellator.setNormal(0.0f, 0.0f, -1.0f);
        instance.addVertex(aabb.minX, aabb.maxY, aabb.minZ);
        instance.addVertex(aabb.maxX, aabb.maxY, aabb.minZ);
        instance.addVertex(aabb.maxX, aabb.minY, aabb.minZ);
        instance.addVertex(aabb.minX, aabb.minY, aabb.minZ);
        Tessellator.setNormal(0.0f, 0.0f, 1.0f);
        instance.addVertex(aabb.minX, aabb.minY, aabb.maxZ);
        instance.addVertex(aabb.maxX, aabb.minY, aabb.maxZ);
        instance.addVertex(aabb.maxX, aabb.maxY, aabb.maxZ);
        instance.addVertex(aabb.minX, aabb.maxY, aabb.maxZ);
        Tessellator.setNormal(0.0f, -1.0f, 0.0f);
        instance.addVertex(aabb.minX, aabb.minY, aabb.minZ);
        instance.addVertex(aabb.maxX, aabb.minY, aabb.minZ);
        instance.addVertex(aabb.maxX, aabb.minY, aabb.maxZ);
        instance.addVertex(aabb.minX, aabb.minY, aabb.maxZ);
        Tessellator.setNormal(0.0f, 1.0f, 0.0f);
        instance.addVertex(aabb.minX, aabb.maxY, aabb.maxZ);
        instance.addVertex(aabb.maxX, aabb.maxY, aabb.maxZ);
        instance.addVertex(aabb.maxX, aabb.maxY, aabb.minZ);
        instance.addVertex(aabb.minX, aabb.maxY, aabb.minZ);
        Tessellator.setNormal(-1.0f, 0.0f, 0.0f);
        instance.addVertex(aabb.minX, aabb.minY, aabb.maxZ);
        instance.addVertex(aabb.minX, aabb.maxY, aabb.maxZ);
        instance.addVertex(aabb.minX, aabb.maxY, aabb.minZ);
        instance.addVertex(aabb.minX, aabb.minY, aabb.minZ);
        Tessellator.setNormal(1.0f, 0.0f, 0.0f);
        instance.addVertex(aabb.maxX, aabb.minY, aabb.minZ);
        instance.addVertex(aabb.maxX, aabb.maxY, aabb.minZ);
        instance.addVertex(aabb.maxX, aabb.maxY, aabb.maxZ);
        instance.addVertex(aabb.maxX, aabb.minY, aabb.maxZ);
        instance.draw();
        GL11.glEnable(3553);
    }
    
    public final void setRenderManager(final RenderManager renderManager) {
        this.renderManager = renderManager;
    }
    
    public final void renderShadow(final Entity entity, final double double2, final double double3, final double double4, double double5) {
        if (this.shadowSize > 0.0f) {
            double5 = this.renderManager.getDistanceToCamera(double2, double3, double4);
            if ((double5 = (1.0f - double5 / 256.0f) * this.shadowOpaque) > 0.0f) {
                final double n = double5;
                final double n2 = double4;
                final double n3 = double3;
                GL11.glEnable(3042);
                RenderEngine.bindTexture(this.renderManager.renderEngine.getTexture("%%/shadow.png"));
                final World worldObj = this.renderManager.worldObj;
                GL11.glDepthMask(false);
                final double shadowSize = this.shadowSize;
                for (int i = (int)(double2 - shadowSize); i <= (int)(double2 + shadowSize); ++i) {
                    for (int j = (int)(n3 - 2.0f); j <= (int)n3; ++j) {
                        for (int k = (int)(n2 - shadowSize); k <= (int)(n2 + shadowSize); ++k) {
                            final int blockId;
                            if ((blockId = worldObj.getBlockId(i, j - 1, k)) > 0 && worldObj.getBlockLightValue(i, j, k) > 3) {
                                final Block block = Block.blocksList[blockId];
                                final double n4 = n3;
                                final double n5 = n2;
                                final int n6 = i;
                                final int n7 = j;
                                final int n8 = k;
                                final double n9 = n;
                                final double n10 = shadowSize;
                                double n11 = n9;
                                final int integer3 = n8;
                                final int integer4 = n7;
                                final int integer5 = n6;
                                double n12 = n5;
                                double n13 = n4;
                                double n14 = double2;
                                final Block block2 = block;
                                final Tessellator instance = Tessellator.instance;
                                double n15;
                                if ((n15 = (n11 - (n13 - integer4) / 2.0f) * 0.5f * this.renderManager.worldObj.getLightBrightness(integer5, integer4, integer3)) >= 0.0f) {
                                    GL11.glColor4d(1.0f, 1.0f, 1.0f, n15);
                                    instance.startDrawingQuads();
                                    n15 = integer5 + block2.minX;
                                    n13 = integer5 + block2.maxX;
                                    final double n16 = integer4 + block2.minY;
                                    final double n17 = integer3 + block2.minZ;
                                    final double n18 = integer3 + block2.maxZ;
                                    final double n19 = (n14 - n15) / 2.0f / n10 + 0.5f;
                                    n14 = (n14 - n13) / 2.0f / n10 + 0.5f;
                                    n11 = (n12 - n17) / 2.0f / n10 + 0.5f;
                                    n12 = (n12 - n18) / 2.0f / n10 + 0.5f;
                                    instance.addVertexWithUV(n15, n16, n17, n19, n11);
                                    instance.addVertexWithUV(n15, n16, n18, n19, n12);
                                    instance.addVertexWithUV(n13, n16, n18, n14, n12);
                                    instance.addVertexWithUV(n13, n16, n17, n14, n11);
                                    instance.draw();
                                }
                            }
                        }
                    }
                }
                GL11.glColor4d(1.0f, 1.0f, 1.0f, 1.0f);
                GL11.glDisable(3042);
                GL11.glDepthMask(true);
            }
        }
        if (entity.fire > 0) {
            final double n = double4;
            final double n2 = double3;
            final double n3 = double2;
            GL11.glDisable(2896);
            final int blockIndexInTexture;
            final int i = ((blockIndexInTexture = Block.fire.blockIndexInTexture) & 0xF) << 4;
            final int n20 = blockIndexInTexture & 0xF0;
            final double shadowSize = i / 256.0f;
            final double n21 = (i + 15.99f) / 256.0f;
            final double n22 = n20 / 256.0f;
            final double n23 = (n20 + 15.99f) / 256.0f;
            GL11.glPushMatrix();
            GL11.glTranslated(n3, n2, n);
            final double n15;
            GL11.glScaled(n15 = entity.width * 1.4f, n15, n15);
            this.loadTexture("/terrain.png");
            final Tessellator instance2 = Tessellator.instance;
            double n18 = 1.0f;
            double n13 = 0.0f;
            double n12 = entity.height / entity.width;
            GL11.glRotated(-this.renderManager.playerViewY, 0.0f, 1.0f, 0.0f);
            GL11.glTranslated(0.0f, 0.0f, 0.4f + (int)n12 * 0.02f);
            GL11.glColor4d(1.0f, 1.0f, 1.0f, 1.0f);
            instance2.startDrawingQuads();
            while (n12 > 0.0f) {
                instance2.addVertexWithUV(n18 - 0.5f, 0.0f - n13, 0.0f, n21, n23);
                instance2.addVertexWithUV(-0.5f, 0.0f - n13, 0.0f, shadowSize, n23);
                instance2.addVertexWithUV(-0.5f, 1.4f - n13, 0.0f, shadowSize, n22);
                instance2.addVertexWithUV(n18 - 0.5f, 1.4f - n13, 0.0f, n21, n22);
                --n12;
                --n13;
                n18 *= 0.9f;
                GL11.glTranslated(0.0f, 0.0f, -0.04f);
            }
            instance2.draw();
            GL11.glPopMatrix();
            GL11.glEnable(2896);
        }
    }
}
