package net.minecraft.game.block;

import net.minecraft.game.block.tileentity.TileEntity;
import net.minecraft.game.block.tileentity.TileEntityFurnace;
import net.minecraft.game.entity.player.EntityPlayer;
import java.util.Random;
import net.minecraft.game.level.World;

public final class BlockFurnace extends BlockContainer {
    private final boolean isActive;
    
    protected BlockFurnace(final int blockID, final boolean status) {
        super(blockID, Material.rock);
        this.isActive = status;
        this.blockIndexInTexture = 45;
    }
    
    @Override
    public final void onBlockAdded(final World world, final int xCoord, final int yCoord, final int zCoord) {
        super.onBlockAdded(world, xCoord, yCoord, zCoord);
        setDefaultDirection(world, xCoord, yCoord, zCoord);
    }
    
    private static void setDefaultDirection(final World world, final int xCoord, final int yCoord, final int zCoord) {
        final int blockId = world.getBlockId(xCoord, yCoord, zCoord - 1);
        final int blockId2 = world.getBlockId(xCoord, yCoord, zCoord + 1);
        final int blockId3 = world.getBlockId(xCoord - 1, yCoord, zCoord);
        final int blockId4 = world.getBlockId(xCoord + 1, yCoord, zCoord);
        int integer4 = 3;
        if (Block.opaqueCubeLookup[blockId] && !Block.opaqueCubeLookup[blockId2]) {
            integer4 = 3;
        }
        if (Block.opaqueCubeLookup[blockId2] && !Block.opaqueCubeLookup[blockId]) {
            integer4 = 2;
        }
        if (Block.opaqueCubeLookup[blockId3] && !Block.opaqueCubeLookup[blockId4]) {
            integer4 = 5;
        }
        if (Block.opaqueCubeLookup[blockId4] && !Block.opaqueCubeLookup[blockId3]) {
            integer4 = 4;
        }
        world.setBlockMetadataWithNotify(xCoord, yCoord, zCoord, integer4);
    }
    
    @Override
    public final int getBlockTexture(final World world, final int xCoord, final int yCoord, final int zCoord, final int blockSide) {
        if (blockSide == 1) {
            return Block.stone.blockIndexInTexture;
        }
        if (blockSide == 0) {
            return Block.stone.blockIndexInTexture;
        }
        byte b;
        if ((b = world.getBlockMetadata(xCoord, yCoord, zCoord)) == 0) {
            setDefaultDirection(world, xCoord, yCoord, zCoord);
            b = world.getBlockMetadata(xCoord, yCoord, zCoord);
        }
        if (blockSide != b) {
            return this.blockIndexInTexture;
        }
        if (this.isActive) {
            return this.blockIndexInTexture + 16;
        }
        return this.blockIndexInTexture - 1;
    }
    
    @Override
    public final void randomDisplayTick(final World world, final int xCoord, final int yCoord, final int zCoord, final Random random) {
        if (!this.isActive) {
            return;
        }
        final byte blockMetadata = world.getBlockMetadata(xCoord, yCoord, zCoord);
        final double n = xCoord + 0.5f;
        final double n2 = yCoord + random.nextDouble() * 6.0f / 16.0f;
        final double n3 = zCoord + 0.5f;
        final double n4 = random.nextDouble() * 0.6f - 0.3f;
        if (blockMetadata == 4) {
            world.spawnParticle("smoke", n - 0.52f, n2, n3 + n4, 0.0f, 0.0f, 0.0f);
            world.spawnParticle("flame", n - 0.52f, n2, n3 + n4, 0.0f, 0.0f, 0.0f);
            return;
        }
        if (blockMetadata == 5) {
            world.spawnParticle("smoke", n + 0.52f, n2, n3 + n4, 0.0f, 0.0f, 0.0f);
            world.spawnParticle("flame", n + 0.52f, n2, n3 + n4, 0.0f, 0.0f, 0.0f);
            return;
        }
        if (blockMetadata == 2) {
            world.spawnParticle("smoke", n + n4, n2, n3 - 0.52f, 0.0f, 0.0f, 0.0f);
            world.spawnParticle("flame", n + n4, n2, n3 - 0.52f, 0.0f, 0.0f, 0.0f);
            return;
        }
        if (blockMetadata == 3) {
            world.spawnParticle("smoke", n + n4, n2, n3 + 0.52f, 0.0f, 0.0f, 0.0f);
            world.spawnParticle("flame", n + n4, n2, n3 + 0.52f, 0.0f, 0.0f, 0.0f);
        }
    }
    
    @Override
    public final int getBlockTextureFromSide(final int blockSide) {
        if (blockSide == 1) {
            return Block.stone.blockID;
        }
        if (blockSide == 0) {
            return Block.stone.blockID;
        }
        if (blockSide == 3) {
            return this.blockIndexInTexture - 1;
        }
        return this.blockIndexInTexture;
    }
    
    @Override
    public final boolean blockActivated(final World world, final int xCoord, final int yCoord, final int zCoord, final EntityPlayer player) {
        player.displayGUIFurnace((TileEntityFurnace)world.getBlockTileEntity(xCoord, yCoord, zCoord));
        return true;
    }
    
    @Override
    protected final TileEntity getBlockEntity() {
        return new TileEntityFurnace();
    }
}
