package net.minecraft.game.entity;

import java.util.List;
import net.minecraft.game.item.ItemStack;
import net.minecraft.game.entity.other.EntityItem;
import com.nbt.NBTTagList;
import com.nbt.NBTTagDouble;
import com.nbt.NBTBase;
import com.nbt.NBTTagCompound;
import net.minecraft.game.entity.player.EntityPlayer;
import net.minecraft.game.block.Material;
import net.minecraft.game.block.StepSound;
import java.util.ArrayList;
import net.minecraft.game.block.Block;
import net.minecraft.game.MathHelper;
import java.util.Random;
import net.minecraft.client.physics.AxisAlignedBB;
import net.minecraft.game.level.World;

public abstract class Entity {
    public boolean preventEntitySpawning;
    protected World worldObj;
    public double prevPosX;
    public double prevPosY;
    public double prevPosZ;
    public double posX;
    public double posY;
    public double posZ;
    public double motionX;
    public double motionY;
    public double motionZ;
    public double rotationYaw;
    public double rotationPitch;
    public double prevRotationYaw;
    public double prevRotationPitch;
    public AxisAlignedBB boundingBox;
    public boolean onGround;
    public boolean isCollidedHorizontally;
    private boolean unknownBool;
    public boolean isDead;
    public double yOffset;
    public double width;
    public double height;
    public double prevDistanceWalkedModified;
    public double distanceWalkedModified;
    protected boolean entityWalks;
    private double fallDistance;
    private int nextStepDistance;
    public double lastTickPosX;
    public double lastTickPosY;
    public double lastTickPosZ;
    private double ySize;
    public double stepHeight;
    public boolean noClip;
    private double entityCollisionReduction;
    protected Random rand;
    public int ticksExisted;
    public int fireResistance;
    public int fire;
    protected int maxAir;
    private boolean inWater;
    public int heartsLife;
    public int air;
    private boolean isFirstUpdate;
    public String skinUrl;
    
    public Entity(final World world) {
        this.preventEntitySpawning = false;
        this.onGround = false;
        this.isCollidedHorizontally = false;
        this.unknownBool = true;
        this.isDead = false;
        this.yOffset = 0.0f;
        this.width = 0.6f;
        this.height = 1.8f;
        this.prevDistanceWalkedModified = 0.0f;
        this.distanceWalkedModified = 0.0f;
        this.entityWalks = true;
        this.fallDistance = 0.0f;
        this.nextStepDistance = 1;
        this.ySize = 0.0f;
        this.stepHeight = 0.0f;
        this.noClip = false;
        this.entityCollisionReduction = 0.0f;
        this.rand = new Random();
        this.ticksExisted = 0;
        this.fireResistance = 1;
        this.fire = 0;
        this.maxAir = 300;
        this.inWater = false;
        this.heartsLife = 0;
        this.air = 300;
        this.isFirstUpdate = true;
        this.worldObj = world;
        this.setPosition(0.0f, 0.0f, 0.0f);
    }
    
    protected void preparePlayerToSpawn() {
        if (this.worldObj == null) {
            return;
        }
        final double posX = this.worldObj.xSpawn + 0.5f;
        double posY = (double)this.worldObj.ySpawn;
        final double posZ = this.worldObj.zSpawn + 0.5f;
        while (posY > 0.0f) {
            this.setPosition(posX, posY, posZ);
            if (this.worldObj.getCollidingBoundingBoxes(this.boundingBox).size() == 0) {
                break;
            }
            ++posY;
        }
        final double motionX = 0.0f;
        this.motionZ = motionX;
        this.motionY = motionX;
        this.motionX = motionX;
        this.rotationYaw = this.worldObj.rotSpawn;
        this.rotationPitch = 0.0f;
    }
    
    public void setEntityDead() {
        this.isDead = true;
    }
    
    protected void setSize(final double width, final double height) {
        this.width = width;
        this.height = height;
    }
    
    public final void setPosition(final double dx, final double dy, final double dz) {
        this.posX = dx;
        this.posY = dy;
        this.posZ = dz;
        final double n = this.width / 2.0f;
        final double n2 = this.height / 2.0f;
        this.boundingBox = new AxisAlignedBB(dx - n, dy - n2, dz - n, dx + n, dy + n2, dz + n);
    }
    
    public void onEntityUpdate() {
        ++this.ticksExisted;
        this.prevDistanceWalkedModified = this.distanceWalkedModified;
        this.prevPosX = this.posX;
        this.prevPosY = this.posY;
        this.prevPosZ = this.posZ;
        this.prevRotationPitch = this.rotationPitch;
        this.prevRotationYaw = this.rotationYaw;
        if (this.handleWaterMovement()) {
            if (!this.inWater && !this.isFirstUpdate) {
                double double3;
                if ((double3 = MathHelper.sqrt_double(this.motionX * this.motionX * 0.2f + this.motionY * this.motionY + this.motionZ * this.motionZ * 0.2f) * 0.2f) > 1.0f) {
                    double3 = 1.0f;
                }
                this.worldObj.playSoundAtEntity(this, "random.splash", double3, 1.0f + (this.rand.nextDouble() - this.rand.nextDouble()) * 0.4f);
                double3 = (double)(int)this.boundingBox.minY;
                for (int n = 0; n < 1.0f + this.width * 20.0f; ++n) {
                    final double n2 = (this.rand.nextDouble() * 2.0f - 1.0f) * this.width;
                    final double n3 = (this.rand.nextDouble() * 2.0f - 1.0f) * this.width;
                    this.worldObj.spawnParticle("bubble", this.posX + n2, double3 + 1.0f, this.posZ + n3, this.motionX, this.motionY - this.rand.nextDouble() * 0.2f, this.motionZ);
                }
                for (int n = 0; n < 1.0f + this.width * 20.0f; ++n) {
                    final double n2 = (this.rand.nextDouble() * 2.0f - 1.0f) * this.width;
                    final double n3 = (this.rand.nextDouble() * 2.0f - 1.0f) * this.width;
                    this.worldObj.spawnParticle("splash", this.posX + n2, double3 + 1.0f, this.posZ + n3, this.motionX, this.motionY, this.motionZ);
                }
            }
            this.fallDistance = 0.0f;
            this.inWater = true;
            this.fire = 0;
        }
        else {
            this.inWater = false;
        }
        if (this.fire > 0) {
            if (this.fire % 20 == 0) {
                this.attackEntityFrom(null, 1);
            }
            --this.fire;
        }
        if (this.handleLavaMovement()) {
            this.attackEntityFrom(null, 10);
            this.fire = 600;
        }
        /*if (this.posX < -8.0f) {
            final double n4 = -(this.posX + 8.0f);
            this.motionX += n4 * 0.001f;
        }
        if (this.posZ < -8.0f) {
            final double n4 = -(this.posZ + 8.0f);
            this.motionZ += n4 * 0.001f;
        }
        if (this.posX > this.worldObj.width + 8.0f) {
            final double n4 = this.posX - this.worldObj.width + 8.0f;
            this.motionX -= n4 * 0.001f;
        }
        if (this.posZ > this.worldObj.length + 8.0f) {
            final double n4 = this.posZ - this.worldObj.length + 8.0f;
            this.motionZ -= n4 * 0.001f;
        }*/
        this.isFirstUpdate = false;
    }
    
    public final boolean isOffsetPositionInLiquid(final double double1, double double2, double double3) {
        final AxisAlignedBB boundingBox = this.boundingBox;
        final double n = double2;
        final double n2 = double3;
        double3 = n;
        double2 = double1;
        final AxisAlignedBB axisAlignedBB = boundingBox;
        final AxisAlignedBB axisAlignedBB2 = new AxisAlignedBB(axisAlignedBB.minX + n2, axisAlignedBB.minY + double3, axisAlignedBB.minZ + n2, axisAlignedBB.maxX + double2, axisAlignedBB.maxY + double3, axisAlignedBB.maxZ + n2);
        return ((List)this.worldObj.getCollidingBoundingBoxes(axisAlignedBB2)).size() <= 0 && !this.worldObj.getIsAnyLiquid(axisAlignedBB2);
    }
    
    public final void moveEntity(double double1, double double2, double double3) {
        if (this.noClip) {
            this.boundingBox.offset(double1, double2, double3);
            this.posX = (this.boundingBox.minX + this.boundingBox.maxX) / 2.0f;
            this.posY = this.boundingBox.minY + this.yOffset - this.ySize;
            this.posZ = (this.boundingBox.minZ + this.boundingBox.maxZ) / 2.0f;
            return;
        }
        final double posX = this.posX;
        final double posZ = this.posZ;
        final double n = double1;
        final double n2 = double2;
        final double n3 = double3;
        final AxisAlignedBB copy = this.boundingBox.copy();
        final ArrayList collidingBoundingBoxes = this.worldObj.getCollidingBoundingBoxes(this.boundingBox.addCoord(double1, double2, double3));
        for (int i = 0; i < ((List)collidingBoundingBoxes).size(); ++i) {
            double2 = ((AxisAlignedBB)((List)collidingBoundingBoxes).get(i)).calculateYOffset(this.boundingBox, double2);
        }
        this.boundingBox.offset(0.0f, double2, 0.0f);
        if (!this.unknownBool && n2 != double2) {
            double2 = (double1 = (double3 = 0.0f));
        }
        final boolean b = this.onGround || (n2 != double2 && n2 < 0.0f);
        for (int j = 0; j < ((List)collidingBoundingBoxes).size(); ++j) {
            double1 = ((AxisAlignedBB)((List)collidingBoundingBoxes).get(j)).calculateXOffset(this.boundingBox, double1);
        }
        this.boundingBox.offset(double1, 0.0f, 0.0f);
        if (!this.unknownBool && n != double1) {
            double2 = (double1 = (double3 = 0.0f));
        }
        for (int j = 0; j < ((List)collidingBoundingBoxes).size(); ++j) {
            double3 = ((AxisAlignedBB)((List)collidingBoundingBoxes).get(j)).calculateZOffset(this.boundingBox, double3);
        }
        this.boundingBox.offset(0.0f, 0.0f, double3);
        if (!this.unknownBool && n3 != double3) {
            double2 = (double1 = (double3 = 0.0f));
        }
        if (this.stepHeight > 0.0f && b && this.ySize < 0.05f && (n != double1 || n3 != double3)) {
            final double n4 = double1;
            final double n5 = double2;
            final double n6 = double3;
            double1 = n;
            double2 = this.stepHeight;
            double3 = n3;
            final AxisAlignedBB copy2 = this.boundingBox.copy();
            this.boundingBox = copy.copy();
            final ArrayList collidingBoundingBoxes2 = this.worldObj.getCollidingBoundingBoxes(this.boundingBox.addCoord(double1, double2, double3));
            for (int k = 0; k < ((List)collidingBoundingBoxes2).size(); ++k) {
                double2 = ((AxisAlignedBB)((List)collidingBoundingBoxes2).get(k)).calculateYOffset(this.boundingBox, double2);
            }
            this.boundingBox.offset(0.0f, double2, 0.0f);
            if (!this.unknownBool && n2 != double2) {
                double2 = (double1 = (double3 = 0.0f));
            }
            for (int k = 0; k < ((List)collidingBoundingBoxes2).size(); ++k) {
                double1 = ((AxisAlignedBB)((List)collidingBoundingBoxes2).get(k)).calculateXOffset(this.boundingBox, double1);
            }
            this.boundingBox.offset(double1, 0.0f, 0.0f);
            if (!this.unknownBool && n != double1) {
                double2 = (double1 = (double3 = 0.0f));
            }
            for (int k = 0; k < ((List)collidingBoundingBoxes2).size(); ++k) {
                double3 = ((AxisAlignedBB)((List)collidingBoundingBoxes2).get(k)).calculateZOffset(this.boundingBox, double3);
            }
            this.boundingBox.offset(0.0f, 0.0f, double3);
            if (!this.unknownBool && n3 != double3) {
                double2 = (double1 = (double3 = 0.0f));
            }
            if (n4 * n4 + n6 * n6 >= double1 * double1 + double3 * double3) {
                double1 = n4;
                double2 = n5;
                double3 = n6;
                this.boundingBox = copy2.copy();
            }
            else {
                this.ySize += 0.5;
            }
        }
        this.posX = (this.boundingBox.minX + this.boundingBox.maxX) / 2.0f;
        this.posY = this.boundingBox.minY + this.yOffset - this.ySize;
        this.posZ = (this.boundingBox.minZ + this.boundingBox.maxZ) / 2.0f;
        this.isCollidedHorizontally = (n != double1 || n3 != double3);
        this.onGround = (n2 != double2 && n2 < 0.0f);
        if (this.onGround) {
            if (this.fallDistance > 0.0f) {
                this.fall(this.fallDistance);
                this.fallDistance = 0.0f;
            }
        }
        else if (double2 < 0.0f) {
            this.fallDistance -= double2;
        }
        if (n != double1) {
            this.motionX = 0.0f;
        }
        if (n2 != double2) {
            this.motionY = 0.0f;
        }
        if (n3 != double3) {
            this.motionZ = 0.0f;
        }
        final double n4 = this.posX - posX;
        final double n5 = this.posZ - posZ;
        this.distanceWalkedModified += (double)(MathHelper.sqrt_double(n4 * n4 + n5 * n5) * 0.6);
        if (this.entityWalks) {
            final int n7 = (int)this.posX;
            final int n8 = (int)(this.posY - 0.2f - this.yOffset);
            final int k = (int)this.posZ;
            final int blockId = this.worldObj.getBlockId(n7, n8, k);
            if (this.distanceWalkedModified > this.nextStepDistance && blockId > 0) {
                ++this.nextStepDistance;
                final StepSound stepSound = Block.blocksList[blockId].stepSound;
                if (!Block.blocksList[blockId].blockMaterial.getIsLiquid()) {
                    this.worldObj.playSoundAtEntity(this, stepSound.stepSoundDir2(), stepSound.soundVolume * 0.15f, stepSound.soundPitch);
                }
                Block.blocksList[blockId].onEntityWalking(this.worldObj, n7, n8, k);
            }
        }
        this.ySize *= 0.4f;
        final boolean handleWaterMovement = this.handleWaterMovement();
        if (this.worldObj.isBoundingBoxBurning(this.boundingBox)) {
            this.dealFireDamage(1);
            if (!handleWaterMovement) {
                ++this.fire;
                if (this.fire == 0) {
                    this.fire = 300;
                }
            }
        }
        else if (this.fire <= 0) {
            this.fire = -this.fireResistance;
        }
        if (handleWaterMovement && this.fire > 0) {
            this.worldObj.playSoundAtEntity(this, "random.fizz", 0.7f, 1.6f + (this.rand.nextDouble() - this.rand.nextDouble()) * 0.4f);
            this.fire = -this.fireResistance;
        }
    }
    
    protected void dealFireDamage(final int integer) {
        this.attackEntityFrom(null, 1);
    }
    
    protected void fall(final double double1) {
    }
    
    public final boolean handleWaterMovement() {
        return this.worldObj.handleMaterialAcceleration(this.boundingBox.expand(0.0f, -0.4f, 0.0f), Material.water);
    }
    
    public final boolean isInsideOfMaterial() {
        final int blockId;
        return (blockId = this.worldObj.getBlockId((int)this.posX, (int)(this.posY + this.getEyeHeight()), (int)this.posZ)) != 0 && Block.blocksList[blockId].blockMaterial == Material.water;
    }
    
    protected double getEyeHeight() {
        return 0.0f;
    }
    
    public final boolean handleLavaMovement() {
        return this.worldObj.handleMaterialAcceleration(this.boundingBox.expand(0.0f, -0.4f, 0.0f), Material.lava);
    }
    
    public void moveFlying(double double1, double double2, double double3) {
        double n;
        if ((n = MathHelper.sqrt_double(double1 * double1 + double2 * double2)) < 0.01f) {
            return;
        }
        if (n < 1.0f) {
            n = 1.0f;
        }
        n = double3 / n;
        double1 *= n;
        double2 *= n;
        double3 = MathHelper.sin(this.rotationYaw * 3.1415927f / 180.0f);
        n = MathHelper.cos(this.rotationYaw * 3.1415927f / 180.0f);
        this.motionX += double1 * n - double2 * double3;
        this.motionZ += double2 * n + double1 * double3;
    }
    
    public double getEntityBrightness(final double double1) {
        return this.worldObj.getLightBrightness((int)this.posX, (int)(this.posY + this.yOffset / 2.0f), (int)this.posZ);
    }
    
    public final void setWorld(final World world) {
        this.worldObj = world;
    }
    
    public final void setPositionAndRotation(final double posX, final double double2, final double posZ, final double rotationYaw, final double rotationPitch) {
        this.posX = posX;
        this.prevPosX = posX;
        final double n = double2 + this.yOffset;
        this.posY = n;
        this.prevPosY = n;
        this.posZ = posZ;
        this.prevPosZ = posZ;
        this.rotationYaw = rotationYaw;
        this.rotationPitch = rotationPitch;
        this.setPosition(this.posX, this.posY, this.posZ);
    }
    
    public final double getDistanceSqToEntity(final Entity entity) {
        final double n = this.posX - entity.posX;
        final double n2 = this.posY - entity.posY;
        final double n3 = this.posZ - entity.posZ;
        return n * n + n2 * n2 + n3 * n3;
    }
    
    public void onCollideWithPlayer(final EntityPlayer player) {
    }
    
    public final void applyEntityCollision(final Entity entity) {
        double motionX = entity.posX - this.posX;
        double motionZ = entity.posZ - this.posZ;
        double sqrt_double;
        if ((sqrt_double = motionX * motionX + motionZ * motionZ) >= 0.01f) {
            sqrt_double = MathHelper.sqrt_double(sqrt_double);
            motionX /= sqrt_double;
            motionZ /= sqrt_double;
            motionX /= sqrt_double;
            motionZ /= sqrt_double;
            motionX *= 0.05f;
            motionZ *= 0.05f;
            motionX = motionX;
            motionZ = motionZ;
            this.addVelocity(-motionX, 0.0f, -motionZ);
            entity.addVelocity(motionX, 0.0f, motionZ);
        }
    }
    
    private void addVelocity(final double motionX, final double double2, final double motionZ) {
        this.motionX += motionX;
        this.motionY = this.motionY;
        this.motionZ += motionZ;
    }
    
    public boolean attackEntityFrom(final Entity entity, final int integer) {
        return false;
    }
    
    public boolean canBeCollidedWith() {
        return false;
    }
    
    public boolean canBePushed() {
        return false;
    }
    
    public String addToPlayerScore() {
        return null;
    }
    
    public final void writeToNBT(final NBTTagCompound nbtTagCompound) {
        final String entityString = this.getEntityString();
        if (this.isDead || entityString == null) {
            return;
        }
        nbtTagCompound.setString("id", entityString);
        nbtTagCompound.setTag("Pos", (NBTBase)newDoubleNBTList(this.posX, this.posY, this.posZ));
        nbtTagCompound.setTag("Motion", (NBTBase)newDoubleNBTList(this.motionX, this.motionY, this.motionZ));
        nbtTagCompound.setTag("Rotation", (NBTBase)newDoubleNBTList(this.rotationYaw, this.rotationPitch));
        nbtTagCompound.setFloat("FallDistance", (float)this.fallDistance);
        nbtTagCompound.setShort("Fire", (short)this.fire);
        nbtTagCompound.setShort("Air", (short)this.air);
        this.writeEntityToNBT(nbtTagCompound);
    }
    
    public final void readFromNBT(final NBTTagCompound nbtTagCompound) {
        final NBTTagList tagList = nbtTagCompound.getTagList("Pos");
        final NBTTagList tagList2 = nbtTagCompound.getTagList("Motion");
        final NBTTagList tagList3 = nbtTagCompound.getTagList("Rotation");
        this.posX = ((NBTTagDouble)tagList.tagAt(0)).doubleValue;
        this.posY = ((NBTTagDouble)tagList.tagAt(1)).doubleValue;
        this.posZ = ((NBTTagDouble)tagList.tagAt(2)).doubleValue;
        this.motionX = ((NBTTagDouble)tagList2.tagAt(0)).doubleValue;
        this.motionY = ((NBTTagDouble)tagList2.tagAt(1)).doubleValue;
        this.motionZ = ((NBTTagDouble)tagList2.tagAt(2)).doubleValue;
        this.rotationYaw = ((NBTTagDouble)tagList3.tagAt(0)).doubleValue;
        this.rotationPitch = ((NBTTagDouble)tagList3.tagAt(1)).doubleValue;
        this.fallDistance = nbtTagCompound.getFloat("FallDistance");
        this.fire = nbtTagCompound.getShort("Fire");
        this.air = nbtTagCompound.getShort("Air");
        this.setPositionAndRotation(this.posX, this.posY, this.posZ, this.rotationYaw, this.rotationPitch);
        this.readEntityFromNBT(nbtTagCompound);
    }
    
    protected abstract String getEntityString();
    
    protected abstract void readEntityFromNBT(final NBTTagCompound nbtTagCompound);
    
    protected abstract void writeEntityToNBT(final NBTTagCompound nbtTagCompound);
    
    private static NBTTagList newDoubleNBTList(double... arr) {
        final NBTTagList list = new NBTTagList();
        for (int length = (arr = arr).length, i = 0; i < length; ++i) {
            list.setTag(new NBTTagDouble(arr[i]));
        }
        return list;
    }
    
    public double getShadowSize() {
        return this.height / 2.0f;
    }
    
    public final EntityItem dropItemWithOffset(final int integer1, final int integer2) {
        return this.entityDropItem(integer1, 1, 0.0f);
    }
    
    public final EntityItem entityDropItem(final int integer1, final int integer2, final double double3) {
        final EntityItem entity;
        (entity = new EntityItem(this.worldObj, this.posX, this.posY + double3, this.posZ, new ItemStack(integer1, integer2))).delayBeforeCanPickup = 10;
        this.worldObj.entityJoinedWorld(entity);
        return entity;
    }
    
    public boolean isEntityAlive() {
        return !this.isDead;
    }
}
