package net.minecraft.game.entity;

import net.minecraft.game.MathHelper;

public final class PathPoint {
    public final int xCoord;
    public final int yCoord;
    public final int zCoord;
    public final int hash;
    int index;
    double totalPathDistance;
    double distanceToNext;
    double distanceToTarget;
    PathPoint previous;
    public boolean isFirst;
    
    public PathPoint(final int xCoord, final int yCoord, final int zCoord) {
        this.index = -1;
        this.isFirst = false;
        this.xCoord = xCoord;
        this.yCoord = yCoord;
        this.zCoord = zCoord;
        this.hash = (xCoord | yCoord << 10 | zCoord << 20);
    }
    
    public final double distanceTo(final PathPoint pathPoint) {
        final double n = (double)(pathPoint.xCoord - this.xCoord);
        final double n2 = (double)(pathPoint.yCoord - this.yCoord);
        final double n3 = (double)(pathPoint.zCoord - this.zCoord);
        return MathHelper.sqrt_double(n * n + n2 * n2 + n3 * n3);
    }
    
    public final boolean equals(final Object object) {
        return ((PathPoint)object).hash == this.hash;
    }
    
    public final int hashCode() {
        return this.hash;
    }
    
    public final boolean isAssigned() {
        return this.index >= 0;
    }
    
    public final String toString() {
        return new StringBuilder().append(this.xCoord).append(", ").append(this.yCoord).append(", ").append(this.zCoord).toString();
    }
}
