package net.minecraft.game.entity.monster;

import net.minecraft.game.item.Item;
import net.minecraft.game.entity.Entity;
import com.nbt.NBTTagCompound;
import net.minecraft.game.level.World;

public class EntityCreeper extends EntityMonster {
    private int timeSinceIgnited;
    private int lastActiveTime;
    private int fuseTime;
    private int creeperState;
    
    public EntityCreeper(final World world) {
        super(world);
        this.fuseTime = 30;
        this.creeperState = -1;
        this.texture = "/mob/creeper.png";
    }
    
    @Override
    protected final void writeEntityToNBT(final NBTTagCompound nbtTagCompound) {
        super.writeEntityToNBT(nbtTagCompound);
    }
    
    @Override
    protected final void readEntityFromNBT(final NBTTagCompound nbtTagCompound) {
        super.readEntityFromNBT(nbtTagCompound);
    }
    
    @Override
    protected final String getEntityString() {
        return "Creeper";
    }
    
    @Override
    protected final void updatePlayerActionState() {
        this.lastActiveTime = this.timeSinceIgnited;
        if (this.timeSinceIgnited > 0 && this.creeperState < 0) {
            --this.timeSinceIgnited;
        }
        if (this.creeperState >= 0) {
            this.creeperState = 2;
        }
        super.updatePlayerActionState();
        if (this.creeperState != 1) {
            this.creeperState = -1;
        }
    }
    
    @Override
    protected final void attackEntity(final Entity entity, final double double2) {
        if ((this.creeperState <= 0 && double2 < 3.0f) || (this.creeperState > 0 && double2 < 7.0f)) {
            if (this.timeSinceIgnited == 0) {
                this.worldObj.playSoundAtEntity((Entity)this, "random.fuse", 1.0f, 0.5f);
            }
            this.creeperState = 1;
            ++this.timeSinceIgnited;
            if (this.timeSinceIgnited == this.fuseTime) {
                this.worldObj.createExplosion(this, this.posX, this.posY, this.posZ, 3.0f);
                this.setEntityDead();
            }
            this.hasAttacked = true;
        }
    }
    

    public final double setCreeperFlashTime(final double double1) {
        return (this.lastActiveTime + (this.timeSinceIgnited - this.lastActiveTime) * double1) / (this.fuseTime - 2);
    }
    
    @Override
    protected final int scoreValue() {
        return Item.gunpowder.shiftedIndex;
    }
}
