package net.minecraft.game.entity.monster;

import com.nbt.NBTTagCompound;
import net.minecraft.game.entity.Entity;
import net.minecraft.game.level.World;
import net.minecraft.game.entity.EntityCreature;

public class EntityMonster extends EntityCreature {
    protected int attackStrength;
    
    public EntityMonster(final World world) {
        super(world);
        this.attackStrength = 2;
        this.health = 20;
    }
    
    @Override
    public void onLivingUpdate() {
        if (this.getEntityBrightness(1.0f) > 0.5f) {
            this.entityAge += 2;
        }
        super.onLivingUpdate();
    }
    
    @Override
    public final void onEntityUpdate() {
        super.onEntityUpdate();
        if (this.worldObj.difficultySetting == 0) {
            this.setEntityDead();
        }
    }
    
    @Override
    protected Entity findPlayerToAttack() {
        if (this.worldObj.player.getDistanceSqToEntity(this) < 256.0f) {
            return this.worldObj.player;
        }
        return null;
    }
    
    @Override
    public final boolean attackEntityFrom(final Entity entity, final int integer) {
        if (super.attackEntityFrom(entity, integer)) {
            if (entity != this) {
                this.playerToAttack = entity;
            }
            return true;
        }
        return false;
    }
    
    @Override
    protected void attackEntity(final Entity entity, final double double2) {
        if (double2 < 2.5 && entity.boundingBox.maxY > this.boundingBox.minY && entity.boundingBox.minY < this.boundingBox.maxY) {
            this.attackTime = 20;
            entity.attackEntityFrom(this, this.attackStrength);
        }
    }
    
    @Override
    protected double getBlockPathWeight(final int integer1, final int integer2, final int integer3) {
        return 0.5f - this.worldObj.getLightBrightness(integer1, integer2, integer3);
    }
    
    @Override
    protected void writeEntityToNBT(final NBTTagCompound nbtTagCompound) {
        super.writeEntityToNBT(nbtTagCompound);
    }
    
    @Override
    protected void readEntityFromNBT(final NBTTagCompound nbtTagCompound) {
        super.readEntityFromNBT(nbtTagCompound);
    }
    
    @Override
    protected String getEntityString() {
        return "Monster";
    }
    
    @Override
    public final boolean getCanSpawnHere(final double double1, final double double2, final double double3) {
        return this.worldObj.getBlockLightValue((int)double1, (int)double2, (int)double3) <= this.rand.nextInt(8) && super.getCanSpawnHere(double1, double2, double3);
    }
}
