package net.minecraft.game.entity.other;

import net.minecraft.game.entity.player.EntityPlayer;
import com.nbt.NBTTagCompound;
import net.minecraft.game.block.Block;
import net.minecraft.game.block.Material;
import net.minecraft.game.level.World;
import net.minecraft.game.item.ItemStack;
import net.minecraft.game.entity.Entity;

public class EntityItem extends Entity {
    public ItemStack item;
    private int unknownEntityItemInt;
    public int age;
    public int delayBeforeCanPickup;
    private int health;
    public double hoverStart;
    
    public EntityItem(final World world, final double double2, final double double3, final double double4, final ItemStack item) {
        super(world);
        this.age = 0;
        this.health = 5;
        this.hoverStart = (double)(Math.random() * 3.141592653589793 * 2.0);
        this.setSize(0.25f, 0.25f);
        this.yOffset = this.height / 2.0f;
        this.setPosition(double2, double3, double4);
        this.item = item;
        this.rotationYaw = (double)(Math.random() * 360.0);
        this.motionX = (double)(Math.random() * 0.20000000298023224 - 0.10000000149011612);
        this.motionY = 0.2f;
        this.motionZ = (double)(Math.random() * 0.20000000298023224 - 0.10000000149011612);
        this.entityWalks = false;
    }
    
    public EntityItem(final World world) {
        super(world);
        this.age = 0;
        this.health = 5;
        this.hoverStart = (double)(Math.random() * 3.141592653589793 * 2.0);
    }
    
    @Override
    public final void onEntityUpdate() {
        super.onEntityUpdate();
        if (this.delayBeforeCanPickup > 0) {
            --this.delayBeforeCanPickup;
        }
        this.prevPosX = this.posX;
        this.prevPosY = this.posY;
        this.prevPosZ = this.posZ;
        this.motionY -= 0.04f;
        if (this.worldObj.getBlockMaterial((int)this.posX, (int)this.posY, (int)this.posZ) == Material.lava) {
            this.motionY = 0.2f;
            this.motionX = (this.rand.nextDouble() - this.rand.nextDouble()) * 0.2f;
            this.motionZ = (this.rand.nextDouble() - this.rand.nextDouble()) * 0.2f;
            this.worldObj.playSoundAtEntity((Entity)this, "random.fizz", 0.4f, 2.0f + this.rand.nextDouble() * 0.4f);
        }
        final double posX = this.posX;
        final double posY = this.posY;
        double posZ = this.posZ;
        double n = posY;
        double motionZ = posX;
        final int integer1 = (int)motionZ;
        int integer2 = (int)n;
        final int integer3 = (int)posZ;
        motionZ -= integer1;
        n -= integer2;
        posZ -= integer3;
        if (Block.opaqueCubeLookup[this.worldObj.getBlockId(integer1, integer2, integer3)]) {
            final boolean b = !Block.opaqueCubeLookup[this.worldObj.getBlockId(integer1 - 1, integer2, integer3)];
            final boolean b2 = !Block.opaqueCubeLookup[this.worldObj.getBlockId(integer1 + 1, integer2, integer3)];
            final boolean b3 = !Block.opaqueCubeLookup[this.worldObj.getBlockId(integer1, integer2 - 1, integer3)];
            final boolean b4 = !Block.opaqueCubeLookup[this.worldObj.getBlockId(integer1, integer2 + 1, integer3)];
            final boolean b5 = !Block.opaqueCubeLookup[this.worldObj.getBlockId(integer1, integer2, integer3 - 1)];
            final boolean b6 = !Block.opaqueCubeLookup[this.worldObj.getBlockId(integer1, integer2, integer3 + 1)];
            integer2 = -1;
            double n2 = 9999.0f;
            if (b && motionZ < 9999.0f) {
                n2 = motionZ;
                integer2 = 0;
            }
            if (b2 && 1.0f - motionZ < n2) {
                n2 = 1.0f - motionZ;
                integer2 = 1;
            }
            if (b3 && n < n2) {
                n2 = n;
                integer2 = 2;
            }
            if (b4 && 1.0f - n < n2) {
                n2 = 1.0f - n;
                integer2 = 3;
            }
            if (b5 && posZ < n2) {
                n2 = posZ;
                integer2 = 4;
            }
            if (b6 && 1.0f - posZ < n2) {
                integer2 = 5;
            }
            motionZ = this.rand.nextDouble() * 0.2f + 0.1f;
            if (integer2 == 0) {
                this.motionX = -motionZ;
            }
            if (integer2 == 1) {
                this.motionX = motionZ;
            }
            if (integer2 == 2) {
                this.motionY = -motionZ;
            }
            if (integer2 == 3) {
                this.motionY = motionZ;
            }
            if (integer2 == 4) {
                this.motionZ = -motionZ;
            }
            if (integer2 == 5) {
                this.motionZ = motionZ;
            }
        }
        this.moveEntity(this.motionX, this.motionY, this.motionZ);
        this.motionX *= 0.98f;
        this.motionY *= 0.98f;
        this.motionZ *= 0.98f;
        if (this.onGround) {
            this.motionX *= 0.7f;
            this.motionZ *= 0.7f;
            this.motionY *= -0.5f;
        }
        ++this.unknownEntityItemInt;
        ++this.age;
        if (this.age >= 6000) {
            this.setEntityDead();
        }
    }
    
    @Override
    protected final void dealFireDamage(final int integer) {
        this.attackEntityFrom(null, 1);
    }
    
    @Override
    public final boolean attackEntityFrom(final Entity entity, final int integer) {
        this.health -= integer;
        if (this.health <= 0) {
            this.setEntityDead();
        }
        return false;
    }
    
    @Override
    protected final void writeEntityToNBT(final NBTTagCompound nbtTagCompound) {
        nbtTagCompound.setShort("Health", (short)(byte)this.health);
        nbtTagCompound.setShort("Age", (short)this.age);
        nbtTagCompound.setCompoundTag("Item", this.item.writeToNBT(new NBTTagCompound()));
    }
    
    @Override
    protected final void readEntityFromNBT(NBTTagCompound nbtTagCompound) {
        this.health = (nbtTagCompound.getShort("Health") & 0xFF);
        this.age = nbtTagCompound.getShort("Age");
        nbtTagCompound = nbtTagCompound.getCompoundTag("Item");
        this.item = new ItemStack(nbtTagCompound);
    }
    
    @Override
    protected final String getEntityString() {
        return "Item";
    }
    
    @Override
    public final void onCollideWithPlayer(final EntityPlayer player) {
        if (this.delayBeforeCanPickup == 0 && player.inventory.storePartialItemStack(this.item)) {
            this.worldObj.playSoundAtEntity((Entity)this, "random.pop", 0.2f, ((this.rand.nextDouble() - this.rand.nextDouble()) * 0.7f + 1.0f) * 2.0f);
            player.onItemPickup(this);
            this.setEntityDead();
        }
    }
}
