package net.minecraft.game.entity.other;

import com.nbt.NBTTagCompound;
import java.util.List;
import net.minecraft.game.block.Material;
import net.minecraft.game.item.ItemStack;
import net.minecraft.game.item.Item;
import net.minecraft.client.physics.AxisAlignedBB;
import java.util.ArrayList;
import net.minecraft.game.level.World;
import net.minecraft.game.entity.Entity;

public class EntityPainting extends Entity {
    private int tickCounter1;
    public int direction;
    private int xPosition;
    private int yPosition;
    private int zPosition;
    public EnumArt art;
    
    public EntityPainting(final World world) {
        super(world);
        this.tickCounter1 = 0;
        this.direction = 0;
        this.yOffset = 0.0f;
        this.setSize(0.5f, 0.5f);
    }
    
    public EntityPainting(final World world, final int xPosition, int yPosition, int zPosition, final int integer5) {
        this(world);
        this.xPosition = xPosition;
        this.yPosition = yPosition;
        this.zPosition = zPosition;
        final ArrayList list = new ArrayList();
        EnumArt[] values;
        EnumArt art;
        for (yPosition = (values = EnumArt.values()).length, zPosition = 0; zPosition < yPosition; ++zPosition) {
            art = values[zPosition];
            this.art = art;
            this.setDirection(integer5);
            if (this.onValidSurface()) {
                ((List)list).add(art);
            }
        }
        if (((List)list).size() > 0) {
            this.art = (EnumArt)((List)list).get(this.rand.nextInt(((List)list).size()));
        }
        this.setDirection(integer5);
    }
    
    private void setDirection(final int direction) {
        this.direction = direction;
        final double n = (double)(direction * 90);
        this.rotationYaw = n;
        this.prevRotationYaw = n;
        double minX = (double)this.art.sizeX;
        double minY = (double)this.art.sizeY;
        double minZ = (double)this.art.sizeX;
        if (direction == 0 || direction == 2) {
            minZ = 0.5f;
        }
        else {
            minX = 0.5f;
        }
        minX /= 32.0f;
        minY /= 32.0f;
        minZ /= 32.0f;
        double maxX = this.xPosition + 0.5f;
        double maxY = this.yPosition + 0.5f;
        double posZ = this.zPosition + 0.5f;
        if (direction == 0) {
            posZ -= 0.5625f;
        }
        if (direction == 1) {
            maxX -= 0.5625f;
        }
        if (direction == 2) {
            posZ += 0.5625f;
        }
        if (direction == 3) {
            maxX += 0.5625f;
        }
        if (direction == 0) {
            maxX -= getArtSize(this.art.sizeX);
        }
        if (direction == 1) {
            posZ += getArtSize(this.art.sizeX);
        }
        if (direction == 2) {
            maxX += getArtSize(this.art.sizeX);
        }
        if (direction == 3) {
            posZ -= getArtSize(this.art.sizeX);
        }
        maxY += getArtSize(this.art.sizeY);
        this.setPosition(maxX, maxY, posZ);
        this.boundingBox = new AxisAlignedBB(maxX - minX, maxY - minY, posZ - minZ, maxX + minX, maxY + minY, posZ + minZ);
        final AxisAlignedBB boundingBox = this.boundingBox;
        final double n2 = 0.00625f;
        final double n3 = 0.00625f;
        double maxZ = 0.00625f;
        maxZ = n3;
        maxZ = n2;
        final AxisAlignedBB axisAlignedBB = boundingBox;
        minX = boundingBox.minX;
        minY = axisAlignedBB.minY;
        minZ = axisAlignedBB.minZ;
        maxX = axisAlignedBB.maxX;
        maxY = axisAlignedBB.maxY;
        maxZ = axisAlignedBB.maxZ;
        maxX -= 0.00625f;
        maxY -= 0.00625f;
        maxZ -= 0.00625f;
        this.boundingBox = new AxisAlignedBB(minX, minY, minZ, maxX, maxY, maxZ);
    }
    
    private static double getArtSize(final int integer) {
        if (integer == 32) {
            return 0.5f;
        }
        if (integer == 64) {
            return 0.5f;
        }
        return 0.0f;
    }
    
    @Override
    public final void onEntityUpdate() {
        if (this.tickCounter1++ == 100 && !this.onValidSurface()) {
            this.tickCounter1 = 0;
            this.setEntityDead();
            this.worldObj.entityJoinedWorld(new EntityItem(this.worldObj, this.posX, this.posY, this.posZ, new ItemStack(Item.painting)));
        }
    }
    
    public final boolean onValidSurface() {
        if (this.worldObj.getCollidingBoundingBoxes(this.boundingBox).size() > 0) {
            return false;
        }
        final int n = this.art.sizeX / 16;
        final int n2 = this.art.sizeY / 16;
        int xPosition = this.xPosition;
        int zPosition = this.zPosition;
        if (this.direction == 0) {
            xPosition = (int)(this.posX - this.art.sizeX / 32.0f);
        }
        if (this.direction == 1) {
            zPosition = (int)(this.posZ - this.art.sizeX / 32.0f);
        }
        if (this.direction == 2) {
            xPosition = (int)(this.posX - this.art.sizeX / 32.0f);
        }
        if (this.direction == 3) {
            zPosition = (int)(this.posZ - this.art.sizeX / 32.0f);
        }
        final int n3 = (int)(this.posY - this.art.sizeY / 32.0f);
        for (int i = 0; i < n; ++i) {
            for (int j = 0; j < n2; ++j) {
                Material material;
                if (this.direction == 0 || this.direction == 2) {
                    material = this.worldObj.getBlockMaterial(xPosition + i, n3 + j, this.zPosition);
                }
                else {
                    material = this.worldObj.getBlockMaterial(this.xPosition, n3 + j, zPosition + i);
                }
                if (!material.isSolid()) {
                    return false;
                }
            }
        }
        final List entitiesWithinAABBExcludingEntity = this.worldObj.blockMap.getEntitiesWithinAABBExcludingEntity(this, this.boundingBox);
        for (int j = 0; j < entitiesWithinAABBExcludingEntity.size(); ++j) {
            if (entitiesWithinAABBExcludingEntity.get(j) instanceof EntityPainting) {
                return false;
            }
        }
        return true;
    }
    
    @Override
    public final boolean canBeCollidedWith() {
        return true;
    }
    
    @Override
    public final boolean attackEntityFrom(final Entity entity, final int integer) {
        this.setEntityDead();
        this.worldObj.entityJoinedWorld(new EntityItem(this.worldObj, this.posX, this.posY, this.posZ, new ItemStack(Item.painting)));
        return true;
    }
    
    @Override
    protected final void writeEntityToNBT(final NBTTagCompound nbtTagCompound) {
        nbtTagCompound.setByte("Dir", (byte)this.direction);
        nbtTagCompound.setString("Motive", this.art.title);
        nbtTagCompound.setInteger("TileX", this.xPosition);
        nbtTagCompound.setInteger("TileY", this.yPosition);
        nbtTagCompound.setInteger("TileZ", this.zPosition);
    }
    
    @Override
    protected final String getEntityString() {
        return "Painting";
    }
    
    @Override
    protected final void readEntityFromNBT(final NBTTagCompound nbtTagCompound) {
        this.direction = nbtTagCompound.getByte("Dir");
        this.xPosition = nbtTagCompound.getInteger("TileX");
        this.yPosition = nbtTagCompound.getInteger("TileY");
        this.zPosition = nbtTagCompound.getInteger("TileZ");
        final String string = nbtTagCompound.getString("Motive");
        EnumArt[] values;
        for (int length = (values = EnumArt.values()).length, i = 0; i < length; ++i) {
            final EnumArt art;
            if ((art = values[i]).title.equals(string)) {
                this.art = art;
            }
        }
        if (this.art == null) {
            this.art = EnumArt.Kebab;
        }
        this.setDirection(this.direction);
    }
}
