package net.minecraft.game.level;

import java.util.Set;
import net.minecraft.game.block.BlockContainer;
import java.util.Collection;
import java.util.TreeSet;
import net.minecraft.client.physics.MovingObjectPosition;
import net.minecraft.game.block.Material;
import net.minecraft.client.renderer.Vec3D;
import net.minecraft.game.MathHelper;
import net.minecraft.game.block.tileentity.TileEntity;
import net.minecraft.client.physics.AxisAlignedBB;
import net.minecraft.game.entity.Entity;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.ArrayList;
import net.minecraft.game.block.Block;
import net.minecraft.game.entity.Pathfinder;
import net.minecraft.game.level.generator.ChunkGen;
import net.minecraft.game.entity.EntityLiving;
import java.util.Random;
import java.util.Map;
import java.util.Objects;
import java.util.List;

public final class World {
    private static double[] lightBrightnessTable;
    public int width = 256;
    public int length = 256;
    public int height = 256;
    //public byte[] blocks;
    public byte[] dataByteArray;
    public String name;
    public String authorName;
    public long createTime;
    public int xSpawn;
    public int ySpawn;
    public int zSpawn;
    public double rotSpawn;
    public int defaultFluid;
    List worldAccesses;
    private List tickList;
    public Map<Integer,Object> map;
    private List list;
    int[] worldIntArray1;
    public Random random;
    private Random rand;
    private int randId;
    public BlockMap blockMap;
    public int getWaterLevel;
    public int getGroundLevel;
    public int cloudHeight = 128;
    public int skyColor;
    public int fogColor;
    public int cloudColor;
    private int worldInt1;
    private int worldInt2;
    public EntityLiving player;
    public boolean survivalWorld;
    public int skyBrightness;
    public int skylightSubtracted;
    public Pathfinder pathFinder;
    private WorldClient worldClient;
    public int timeOfDay;
    private static short worldShort;
    private short[] blocksShortArray;
    private int[] worldIntArray2;
    private int[] worldIntArray3;
    public int difficultySetting;
    private final ChunkGen chunkGen = new ChunkGen();
    
    public HashMap<ChunkCoord,Chunk> chunks = new HashMap<>();
    
    public World() {
        this.defaultFluid = Block.waterMoving.blockID;
        this.worldAccesses = (List)new ArrayList();
        this.tickList = (List)new LinkedList();
        this.map = (Map)new HashMap();
        this.list = (List)new ArrayList();
        this.random = new Random();
        this.rand = new Random();
        this.randId = this.random.nextInt();
        this.worldInt1 = 0;
        this.worldInt2 = 0;
        this.survivalWorld = true;
        this.skyBrightness = 15;
        this.skylightSubtracted = 15;
        this.pathFinder = new Pathfinder(this);
        this.timeOfDay = 0;
        this.blocksShortArray = new short[1048576];
        this.worldIntArray2 = new int[1048576];
        this.worldIntArray3 = new int[1048576];
        this.difficultySetting = 2;
        this.skyColor = 10079487;
        this.fogColor = 16777215;
        this.cloudColor = 16777215;
    }
    
    public final void load() {
        this.worldAccesses = (List)new ArrayList();
        Arrays.fill(this.worldIntArray1 = new int[this.width * this.length], this.height);
        this.worldClient.updateSkylight(0, 0, this.width, this.length);
        this.random = new Random();
        this.randId = this.random.nextInt();
        this.tickList = (List)new ArrayList();
        if (this.blockMap == null) {
            this.blockMap = new BlockMap(this);
        }
    }
    
    public final void generate(int integer1, int integer2, int integer3, final byte[] arr4, byte[] var_5_0C) {
        /*if (var_5_0C != null && var_5_0C.length == 0) {
            var_5_0C = null;
        }
        this.width = integer1;
        this.length = integer3;
        this.height = integer2;
        this.blocks = arr4;
        int i;
        int j;
        int n;
        for (integer2 = 0; integer2 < this.width; ++integer2) {
            for (i = 0; i < this.length; ++i) {
                for (j = 0; j < this.height; ++j) {
                    n = 0;
                    if (j <= 1 && j < this.getGroundLevel - 1 && arr4[((j + 1) * this.length + i) * this.width + integer2] == 0) {
                        n = Block.lavaStill.blockID;
                    }
                    else if (j < this.getGroundLevel - 1) {
                        n = Block.bedrock.blockID;
                    }
                    else if (j < this.getGroundLevel) {
                        if (this.getGroundLevel > this.getWaterLevel && this.defaultFluid == Block.waterMoving.blockID) {
                            n = Block.grass.blockID;
                        }
                        else {
                            n = Block.dirt.blockID;
                        }
                    }
                    else if (j < this.getWaterLevel) {
                        n = this.defaultFluid;
                    }
                    arr4[(j * this.length + i) * this.width + integer2] = (byte)n;
                    if (j == 1 && integer2 != 0 && i != 0 && integer2 != this.width - 1 && i != this.length - 1) {
                        j = this.height - 2;
                    }
                }
            }
        }
        Arrays.fill(this.worldIntArray1 = new int[integer1 * integer3], this.height);
        if (var_5_0C == null) {
            this.dataByteArray = new byte[arr4.length];
            this.worldClient = new WorldClient(this);
            integer1 = 1;
            integer2 = this.skylightSubtracted;
            int k;
            int l;
            byte b;
            for (integer3 = 0; integer3 < this.width; ++integer3) {
                for (k = 0; k < this.length; ++k) {
                    for (l = this.height - 1; l > 0 && Block.lightOpacity[this.getBlockId(integer3, l, k)] == 0; --l) {}
                    this.worldIntArray1[integer3 + k * this.width] = l + 1;
                    for (l = 0; l < this.height; ++l) {
                        i = (l * this.length + k) * this.width + integer3;
                        j = this.worldIntArray1[integer3 + k * this.width];
                        j = ((l >= j) ? integer2 : 0);
                        b = this.blocks[i];
                        if (j < Block.lightValue[b]) {
                            j = Block.lightValue[b];
                        }
                        this.dataByteArray[i] = (byte)((this.dataByteArray[i] & 0xF0) + j);
                    }
                }
            }
            this.worldClient.updateBlockLight(0, 0, 0, this.width, this.height, this.length);
        }
        else {
            this.dataByteArray = var_5_0C;
            this.worldClient = new WorldClient(this);
        }*/
    	this.worldClient = new WorldClient(this);
        for (integer2 = 0; integer2 < this.worldAccesses.size(); ++integer2) {
            ((IWorldAccess)this.worldAccesses.get(integer2)).loadRenderers();
        }
        this.tickList.clear();
        this.findSpawn();
        this.load();
        System.gc();
    }
    
    public final void findSpawn() {
        /*final Random random = new Random();
        int n = 0;
    Label_0010:
        while (true) {
            ++n;
            final int xSpawn = 64;
            final int zSpawn = 64;
            final int ySpawn = 1;
            if (n == 1000000) {
                this.xSpawn = xSpawn;
                this.ySpawn = this.height + 100;
                this.zSpawn = zSpawn;
                this.rotSpawn = 180.0f;
                return;
            }
            if (ySpawn >= 4 && ySpawn > this.getWaterLevel) {
                for (int i = xSpawn - 3; i <= xSpawn + 3; ++i) {
                    for (int j = ySpawn - 1; j <= ySpawn + 2; ++j) {
                        for (int k = zSpawn - 3 - 2; k <= zSpawn + 3; ++k) {
                            if (this.getBlockMaterial(i, j, k).isSolid()) {
                                continue Label_0010;
                            }
                        }
                    }
                }
                int i = ySpawn - 2;
                for (int j = xSpawn - 3; j <= xSpawn + 3; ++j) {
                    for (int k = zSpawn - 3 - 2; k <= zSpawn + 3; ++k) {
                        if (!Block.opaqueCubeLookup[this.getBlockId(j, i, k)]) {
                            continue Label_0010;
                        }
                    }
                }
                this.xSpawn = xSpawn;
                this.ySpawn = ySpawn;
                this.zSpawn = zSpawn;
                this.rotSpawn = 180.0f;
                return;
            }
        }*/
    	this.xSpawn = 0;
        this.ySpawn = 2;
        this.zSpawn = 0;
    }
    
    public final void addWorldAccess(final IWorldAccess iWorldAccess) {
        for (int i = 0; i < this.blockMap.all.size(); ++i) {
            iWorldAccess.obtainEntitySkin((Entity)this.blockMap.all.get(i));
        }
        this.worldAccesses.add(iWorldAccess);
    }
    
    public final void finalize() {
    }
    
    public final void removeWorldAccess(final IWorldAccess iWorldAccess) {
        this.worldAccesses.remove(iWorldAccess);
    }
    
    public final ArrayList getCollidingBoundingBoxes(final AxisAlignedBB aabb) {
        final ArrayList list = new ArrayList();
        int i = (int)aabb.minX;
        final int n = (int)aabb.maxX + 1;
        int n2 = (int)aabb.minY;
        final int n3 = (int)aabb.maxY + 1;
        int n4 = (int)aabb.minZ;
        final int n5 = (int)aabb.maxZ + 1;
        if (aabb.minX < 0.0f) {
            --i;
        }
        if (aabb.minY < 0.0f) {
            --n2;
        }
        if (aabb.minZ < 0.0f) {
            --n4;
        }
        for (i = i; i < n; ++i) {
            for (int j = n2; j < n3; ++j) {
                for (int k = n4; k < n5; ++k) {
                    final Block block;
                    if ((block = Block.blocksList[this.getBlockId(i, j, k)]) != null) {
                        final AxisAlignedBB collisionBoundingBoxFromPool;
                        if ((collisionBoundingBoxFromPool = block.getCollisionBoundingBoxFromPool(i, j, k)) != null && aabb.intersectsWith(collisionBoundingBoxFromPool)) {
                            list.add(collisionBoundingBoxFromPool);
                        }
                    }
                    else {
                        final AxisAlignedBB collisionBoundingBoxFromPool2;
                        if (this.getGroundLevel < 0 && (j < this.getGroundLevel || j < this.getWaterLevel) && (collisionBoundingBoxFromPool2 = Block.bedrock.getCollisionBoundingBoxFromPool(i, j, k)) != null && aabb.intersectsWith(collisionBoundingBoxFromPool2)) {
                            list.add(collisionBoundingBoxFromPool2);
                        }
                    }
                }
            }
        }
        return list;
    }
    
    public final void swap(final int integer1, final int integer2, final int integer3, final int integer4, final int integer5, final int integer6) {
        final int blockId = this.getBlockId(integer1, integer2, integer3);
        final int blockId2 = this.getBlockId(integer4, integer5, integer6);
        this.setTileNoNeighborChange(integer1, integer2, integer3, blockId2);
        this.setTileNoNeighborChange(integer4, integer5, integer6, blockId);
        this.notifyBlocksOfNeighborChange(integer1, integer2, integer3, blockId2);
        this.notifyBlocksOfNeighborChange(integer4, integer5, integer6, blockId);
    }
    
    public final boolean setTileNoNeighborChange(final int integer1, final int integer2, final int integer3, int integer4) {
        if (integer4 == this.getBlockId(integer1, integer2, integer3)) {
            return false;
        }
        final byte b = (byte) this.getBlockId(integer1, integer2, integer3);
        Chunk chunk = getChunk(MathHelper.floor_double(integer1/(double)Chunk.WIDTH),MathHelper.floor_double(integer2/(double)Chunk.HEIGHT),MathHelper.floor_double((double)integer3/Chunk.WIDTH));
        chunk.setBlock(integer1&Chunk.WIDTH-1,integer2&Chunk.HEIGHT-1,integer3&Chunk.WIDTH-1, integer4);
        this.setBlockMetadataWithNotify(integer1, integer2, integer3, 0);
        if (b != 0) {
            Block.blocksList[b].onBlockRemoval(this, integer1, integer2, integer3);
        }
        if (integer4 != 0) {
            Block.blocksList[integer4].onBlockAdded(this, integer1, integer2, integer3);
        }
        if (Block.lightOpacity[b] != Block.lightOpacity[integer4] || Block.lightValue[b] != 0 || Block.lightValue[integer4] != 0) {
            this.worldClient.updateSkylight(integer1, integer3, 1, 1);
            this.worldClient.updateBlockLight(integer1, integer2, integer3, integer1 + 1, integer2 + 1, integer3 + 1);
        }
        for (integer4 = 0; integer4 < this.worldAccesses.size(); ++integer4) {
            ((IWorldAccess)this.worldAccesses.get(integer4)).markBlockAndNeighborsNeedsUpdate(integer1, integer2, integer3);
        }
        return true;
    }
    
    public Chunk getChunk(int x,int y,int z) {
		Chunk chunk = chunks.get(new ChunkCoord(x,y,z));
		if(chunk==null) {
			if(!chunks.containsKey(new ChunkCoord(x,0,z))) {
				Chunk[] chunkArray = chunkGen.provideChunk(x, z);
				for (int i = 0; i < chunkArray.length; i++) {
					Chunk chunk2 = chunkArray[i];
					chunks.put(new ChunkCoord(x,i,z), chunk2);
					if(y == i) chunk = chunk2;
				}
			}
			if(chunk==null) chunk = new Chunk(new byte[Chunk.WIDTH][Chunk.HEIGHT][Chunk.WIDTH]);
			chunks.put(new ChunkCoord(x,y,z), chunk);
		}
		return chunk;
	}

	public final boolean setBlockWithNotify(final int integer1, final int integer2, final int integer3, final int integer4) {
        if (this.setTileNoNeighborChange(integer1, integer2, integer3, integer4)) {
            this.notifyBlocksOfNeighborChange(integer1, integer2, integer3, integer4);
            return true;
        }
        return false;
    }
    
    public final void notifyBlocksOfNeighborChange(final int integer1, final int integer2, final int integer3, final int integer4) {
        this.updateTile(integer1 - 1, integer2, integer3, integer4);
        this.updateTile(integer1 + 1, integer2, integer3, integer4);
        this.updateTile(integer1, integer2 - 1, integer3, integer4);
        this.updateTile(integer1, integer2 + 1, integer3, integer4);
        this.updateTile(integer1, integer2, integer3 - 1, integer4);
        this.updateTile(integer1, integer2, integer3 + 1, integer4);
    }
    
    public final boolean setTileNoUpdate(final int integer1, final int integer2, final int integer3, final int integer4) {
        if (integer4 == this.getBlockId(integer1, integer2, integer3)) {
            return false;
        }
        Chunk chunk = getChunk(MathHelper.floor_double(integer1/(double)Chunk.WIDTH),MathHelper.floor_double(integer2/(double)Chunk.HEIGHT),MathHelper.floor_double(integer3/(double)Chunk.WIDTH));
        chunk.setBlock(integer1&Chunk.WIDTH-1,integer2&Chunk.HEIGHT-1,integer3&Chunk.WIDTH-1, integer4);
        this.worldClient.updateBlockLight(integer1, integer2, integer3, integer1 + 1, integer2 + 1, integer3 + 1);
        return true;
    }
    
    private void updateTile(final int integer1, final int integer2, final int integer3, final int integer4) {
        final Block block;
        if ((block = Block.blocksList[this.getBlockId(integer1, integer2, integer3)]) != null) {
            block.onNeighborBlockChange(this, integer1, integer2, integer3, integer4);
        }
    }
    
    public final int getBlockId(int integer1, int integer2, int integer3) {
    	Chunk chunk = getChunk(MathHelper.floor_double(integer1/(double)Chunk.WIDTH),MathHelper.floor_double(integer2/(double)Chunk.HEIGHT),MathHelper.floor_double(integer3/(double)Chunk.WIDTH));
        return chunk.getBlock(integer1&Chunk.WIDTH-1,integer2&Chunk.HEIGHT-1,integer3&Chunk.WIDTH-1) & 0xFF;
    }
    
    public final boolean isBlockNormalCube(final int integer1, final int integer2, final int integer3) {
        final Block block;
        return (block = Block.blocksList[this.getBlockId(integer1, integer2, integer3)]) != null && block.isOpaqueCube();
    }
    
    public final void levelEntities() {
        this.blockMap.tickAll();
        for (int i = 0; i < this.list.size(); ++i) {
            ((TileEntity)this.list.get(i)).updateEntity();
        }
    }
    
    public final void tickEntities() {
        this.worldClient.tick();
    }
    
    public final double getStarBrightness(double double1) {
        double1 = this.getCelestialAngle(double1);
        if ((double1 = 1.0f - (MathHelper.cos(double1 * 3.1415927f * 2.0f) * 2.0f + 0.75f)) < 0.0f) {
            double1 = 0.0f;
        }
        if (double1 > 1.0f) {
            double1 = 1.0f;
        }
        return double1 * double1 * 0.5f;
    }
    
    public final Vec3D getSkyColor(double double1) {
        if ((double1 = MathHelper.cos((double1 = this.getCelestialAngle(double1)) * 3.1415927f * 2.0f) * 2.0f + 0.5f) < 0.0f) {
            double1 = 0.0f;
        }
        if (double1 > 1.0f) {
            double1 = 1.0f;
        }
        double double2 = (this.skyColor >> 16 & 0xFF) / 255.0f;
        double double3 = (this.skyColor >> 8 & 0xFF) / 255.0f;
        double double4 = (this.skyColor & 0xFF) / 255.0f;
        double2 *= double1;
        double3 *= double1;
        double4 *= double1;
        return new Vec3D(double2, double3, double4);
    }
    
    public final double getCelestialAngle(double double1) {
        if (this.skyBrightness > 15) {
            return 0.0f;
        }
        return (this.timeOfDay + double1) / 24000.0f - 0.15f;
    }
    
    public final Vec3D getFogColor(double double1) {
        if ((double1 = MathHelper.cos((double1 = this.getCelestialAngle(double1)) * 3.1415927f * 2.0f) * 2.0f + 0.5f) < 0.0f) {
            double1 = 0.0f;
        }
        if (double1 > 1.0f) {
            double1 = 1.0f;
        }
        double double2 = (this.fogColor >> 16 & 0xFF) / 255.0f;
        double double3 = (this.fogColor >> 8 & 0xFF) / 255.0f;
        double double4 = (this.fogColor & 0xFF) / 255.0f;
        double2 *= double1 * 0.94f + 0.06f;
        double3 *= double1 * 0.94f + 0.06f;
        double4 *= double1 * 0.91f + 0.09f;
        return new Vec3D(double2, double3, double4);
    }
    
    public final Vec3D getCloudColor(double double1) {
        if ((double1 = MathHelper.cos((double1 = this.getCelestialAngle(double1)) * 3.1415927f * 2.0f) * 2.0f + 0.5f) < 0.0f) {
            double1 = 0.0f;
        }
        if (double1 > 1.0f) {
            double1 = 1.0f;
        }
        double double2 = (this.cloudColor >> 16 & 0xFF) / 255.0f;
        double double3 = (this.cloudColor >> 8 & 0xFF) / 255.0f;
        double double4 = (this.cloudColor & 0xFF) / 255.0f;
        double2 *= double1 * 0.9f + 0.1f;
        double3 *= double1 * 0.9f + 0.1f;
        double4 *= double1 * 0.85f + 0.15f;
        return new Vec3D(double2, double3, double4);
    }
    
    public final int getSkyBrightness() {
        double celestialAngle;
        if ((celestialAngle = MathHelper.cos((celestialAngle = this.getCelestialAngle(1.0f)) * 3.1415927f * 2.0f) * 1.5f + 0.5f) < 0.0f) {
            celestialAngle = 0.0f;
        }
        if (celestialAngle > 1.0f) {
            celestialAngle = 1.0f;
        }
        int n;
        if ((n = (int)(celestialAngle * (15 * this.skyBrightness / 15.0f - 4.0f) + 4.0f)) > 15) {
            n = 15;
        }
        if (n < 4) {
            n = 4;
        }
        return n;
    }
    
    public final void restartTimeOfDay() {
        ++this.timeOfDay;
        if (this.timeOfDay == 24000) {
            this.timeOfDay = 0;
        }
        /*int skyBrightness = this.getSkyBrightness();
        if (this.skylightSubtracted > skyBrightness) {
            this.updateChunkLight(this.skylightSubtracted - 1);
        }
        if (this.skylightSubtracted < skyBrightness) {
            this.updateChunkLight(this.skylightSubtracted + 1);
        }
        ++this.worldInt2;
        skyBrightness = 1;
        int n = 1;
        while (1 << skyBrightness < this.width) {
            ++skyBrightness;
        }
        while (1 << n < this.length) {
            ++n;
        }
        final int n2 = this.length - 1;
        final int n3 = this.width - 1;
        final int n4 = this.height - 1;*/
        int size;
        if ((size = this.tickList.size()) > 200) {
            size = 200;
        }
        for (int i = 0; i < size; ++i) {
            final NextTickListEntry nextTickListEntry;
            if ((nextTickListEntry = (NextTickListEntry)this.tickList.get(i)).ticks > 0) {
                --nextTickListEntry.ticks;
            }
            else {
                final int x = nextTickListEntry.x;
                final int y = nextTickListEntry.y;
                final int z = nextTickListEntry.z;
                final int n5 = y;
                final int zCoord = x;
                final int b;
                if ((b=getBlockId(x,y,z)) == nextTickListEntry.block && b > 0) {
                    Block.blocksList[b].updateTick(this, nextTickListEntry.x, nextTickListEntry.y, nextTickListEntry.z, this.random);
                }
                this.tickList.remove(i--);
            }
        }
        /*this.worldInt1 += this.width * this.length * this.height;
        size = this.worldInt1 / 200;
        this.worldInt1 -= size * 200;
        for (int i = 0; i < size; ++i) {
            this.randId = this.randId * 3 + 1013904223;
            int yCoord;
            final int xCoord = (yCoord = this.randId >> 2) & n3;
            final int zCoord = yCoord >> skyBrightness & n2;
            yCoord = (yCoord >> skyBrightness + n & n4);
            final byte b2 = this.blocks[(yCoord * this.length + zCoord) * this.width + xCoord];
            if (Block.tickOnLoad[b2]) {
                Block.blocksList[b2].updateTick(this, xCoord, yCoord, zCoord, this.random);
            }
        }*/
    }
    
    public final int entitiesInLevelList(final Class class1) {
        int n = 0;
        for (int i = 0; i < this.blockMap.all.size(); ++i) {
            if (class1.isAssignableFrom(((Entity)this.blockMap.all.get(i)).getClass())) {
                ++n;
            }
        }
        return n;
    }
    
    public final int getGroundLevel() {
        return this.getGroundLevel;
    }
    
    public final int getWaterLevel() {
        return this.getWaterLevel;
    }
    
    public final boolean getIsAnyLiquid(final AxisAlignedBB aabb) {
        int i = (int)aabb.minX;
        int width = (int)aabb.maxX + 1;
        int n = (int)aabb.minY;
        int height = (int)aabb.maxY + 1;
        int n2 = (int)aabb.minZ;
        int length = (int)aabb.maxZ + 1;
        if (aabb.minX < 0.0f) {
            --i;
        }
        if (aabb.minY < 0.0f) {
            --n;
        }
        if (aabb.minZ < 0.0f) {
            --n2;
        }
        if (i < 0) {
            i = 0;
        }
        if (n < 0) {
            n = 0;
        }
        if (n2 < 0) {
            n2 = 0;
        }
        if (width > this.width) {
            width = this.width;
        }
        if (height > this.height) {
            height = this.height;
        }
        if (length > this.length) {
            length = this.length;
        }
        for (int j = i; j < width; ++j) {
            for (i = n; i < height; ++i) {
                for (int k = n2; k < length; ++k) {
                    final Block block;
                    if ((block = Block.blocksList[this.getBlockId(j, i, k)]) != null && block.blockMaterial.getIsLiquid()) {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    public final boolean isBoundingBoxBurning(final AxisAlignedBB aabb) {
        int i = (int)aabb.minX;
        final int n = (int)aabb.maxX + 1;
        final int n2 = (int)aabb.minY;
        final int n3 = (int)aabb.maxY + 1;
        final int n4 = (int)aabb.minZ;
        final int n5 = (int)aabb.maxZ + 1;
        for (i = i; i < n; ++i) {
            for (int j = n2; j < n3; ++j) {
                for (int k = n4; k < n5; ++k) {
                    final int blockId;
                    if ((blockId = this.getBlockId(i, j, k)) == Block.fire.blockID || blockId == Block.lavaMoving.blockID || blockId == Block.lavaStill.blockID) {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    public final boolean handleMaterialAcceleration(final AxisAlignedBB aabb, final Material material) {
        int i = (int)aabb.minX;
        final int n = (int)aabb.maxX + 1;
        final int n2 = (int)aabb.minY;
        final int n3 = (int)aabb.maxY + 1;
        final int n4 = (int)aabb.minZ;
        final int n5 = (int)aabb.maxZ + 1;
        for (i = i; i < n; ++i) {
            for (int j = n2; j < n3; ++j) {
                for (int k = n4; k < n5; ++k) {
                    final Block block;
                    if ((block = Block.blocksList[this.getBlockId(i, j, k)]) != null && block.blockMaterial == material) {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    public final void scheduleBlockUpdate(final int integer1, final int integer2, int integer3, final int integer4) {
        final NextTickListEntry nextTickListEntry = new NextTickListEntry(integer1, integer2, integer3, integer4);
        if (integer4 > 0) {
            final NextTickListEntry nextTickListEntry2 = nextTickListEntry;
            integer3 = Block.blocksList[integer4].tickRate();
            nextTickListEntry2.ticks = integer3;
        }
        this.tickList.add(nextTickListEntry);
    }
    
    public final boolean checkIfAABBIsClear1(final AxisAlignedBB aabb) {
        return this.blockMap.getEntitiesWithinAABBExcludingEntity(null, aabb).size() == 0;
    }
    
    public final boolean checkIfAABBIsClear(final AxisAlignedBB aabb) {
        final List entitiesWithinAABBExcludingEntity = this.blockMap.getEntitiesWithinAABBExcludingEntity(null, aabb);
        for (int i = 0; i < entitiesWithinAABBExcludingEntity.size(); ++i) {
            if (((Entity)entitiesWithinAABBExcludingEntity.get(i)).preventEntitySpawning) {
                return false;
            }
        }
        return true;
    }
    
    public final List getEntitiesWithinAABBExcludingEntity(final Entity entity, final AxisAlignedBB aabb) {
        return this.blockMap.getEntitiesWithinAABBExcludingEntity(entity, aabb);
    }
    
    public final boolean isSolid(final double double1, final double double2, final double double3, final double double4) {
        return this.isSolid(double1 - 0.1f, double2 - 0.1f, double3 - 0.1f) || this.isSolid(double1 - 0.1f, double2 - 0.1f, double3 + 0.1f) || this.isSolid(double1 - 0.1f, double2 + 0.1f, double3 - 0.1f) || this.isSolid(double1 - 0.1f, double2 + 0.1f, double3 + 0.1f) || this.isSolid(double1 + 0.1f, double2 - 0.1f, double3 - 0.1f) || this.isSolid(double1 + 0.1f, double2 - 0.1f, double3 + 0.1f) || this.isSolid(double1 + 0.1f, double2 + 0.1f, double3 - 0.1f) || this.isSolid(double1 + 0.1f, double2 + 0.1f, double3 + 0.1f);
    }
    
    private boolean isSolid(final double double1, final double double2, final double double3) {
        final int blockId;
        return (blockId = this.getBlockId((int)double1, (int)double2, (int)double3)) > 0 && Block.blocksList[blockId].isOpaqueCube();
    }
    
    private int getHighestTile(final int integer1, final int integer2) {
        int height;
        for (height = this.height; (this.getBlockId(integer1, height - 1, integer2) == 0 || Block.blocksList[this.getBlockId(integer1, height - 1, integer2)].blockMaterial == Material.air) && height > 0; --height) {}
        return height;
    }
    
    public final void setSpawnPos(final int integer1, final int integer2, final int integer3, final double double4) {
        this.xSpawn = integer1;
        this.ySpawn = integer2;
        this.zSpawn = integer3;
        this.rotSpawn = double4;
    }
    
    public final double getLightBrightness(final int integer1, final int integer2, final int integer3) {
        return World.lightBrightnessTable[this.getBlockLightValue(integer1, integer2, integer3)];
    }
    
    public final byte getBlockLightValue(int integer1, int integer2, int integer3) {
        /*if (integer1 < 0) {
            integer1 = 0;
        }
        else if (integer1 >= this.width) {
            integer1 = this.width - 1;
        }
        if (integer2 < 0) {
            integer2 = 0;
        }
        else if (integer2 >= this.height) {
            integer2 = this.height - 1;
        }
        if (integer3 < 0) {
            integer3 = 0;
        }
        else if (integer3 >= this.length) {
            integer3 = this.length - 1;
        }
        if (this.blocks[(integer2 * this.length + integer3) * this.width + integer1] != Block.stairSingle.blockID) {
            return (byte)(this.dataByteArray[(integer2 * this.length + integer3) * this.width + integer1] & 0xF);
        }
        if (integer2 < this.height - 1) {
            return (byte)(this.dataByteArray[((integer2 + 1) * this.length + integer3) * this.width + integer1] & 0xF);
        }*/
        return 15;
    }
    
    public final byte getBlockMetadata(int integer1, int integer2, int integer3) {
        return 0;
    }
    
    public final void setBlockMetadataWithNotify(int integer1, int integer2, int integer3, int integer4) {
        /*if (integer1 < 0) {
            integer1 = 0;
        }
        else if (integer1 >= this.width) {
            integer1 = this.width - 1;
        }
        if (integer2 < 0) {
            integer2 = 0;
        }
        else if (integer2 >= this.height) {
            integer2 = this.height - 1;
        }
        if (integer3 < 0) {
            integer3 = 0;
        }
        else if (integer3 >= this.length) {
            integer3 = this.length - 1;
        }
        this.dataByteArray[(integer2 * this.length + integer3) * this.width + integer1] = (byte)((this.dataByteArray[(integer2 * this.length + integer3) * this.width + integer1] & 0xF) + (integer4 << 4));*/
        for (integer4 = 0; integer4 < this.worldAccesses.size(); ++integer4) {
            ((IWorldAccess)this.worldAccesses.get(integer4)).markBlockAndNeighborsNeedsUpdate(integer1, integer2, integer3);
        }
    }
    
    public final Material getBlockMaterial(int integer1, final int integer2, final int integer3) {
        if ((integer1 = this.getBlockId(integer1, integer2, integer3)) == 0) {
            return Material.air;
        }
        return Block.blocksList[integer1].blockMaterial;
    }
    
    public final boolean isWater(int integer1, final int integer2, final int integer3) {
        return (integer1 = this.getBlockId(integer1, integer2, integer3)) > 0 && Block.blocksList[integer1].blockMaterial == Material.water;
    }
    
    public final MovingObjectPosition rayTraceBlocks(final Vec3D vec3D, final Vec3D vec3D2) {
        if (Double.isNaN(vec3D.xCoord) || Double.isNaN(vec3D.yCoord) || Double.isNaN(vec3D.zCoord)) {
            return null;
        }
        if (Double.isNaN(vec3D2.xCoord) || Double.isNaN(vec3D2.yCoord) || Double.isNaN(vec3D2.zCoord)) {
            return null;
        }
        final int floor_double = MathHelper.floor_double(vec3D2.xCoord);
        final int floor_double2 = MathHelper.floor_double(vec3D2.yCoord);
        final int floor_double3 = MathHelper.floor_double(vec3D2.zCoord);
        int floor_double4 = MathHelper.floor_double(vec3D.xCoord);
        int floor_double5 = MathHelper.floor_double(vec3D.yCoord);
        int floor_double6 = MathHelper.floor_double(vec3D.zCoord);
        int n = 20;
        while (n-- >= 0) {
            if (Double.isNaN(vec3D.xCoord) || Double.isNaN(vec3D.yCoord) || Double.isNaN(vec3D.zCoord)) {
                return null;
            }
            if (floor_double4 == floor_double && floor_double5 == floor_double2 && floor_double6 == floor_double3) {
                return null;
            }
            double xCoord = 999.0f;
            double yCoord = 999.0f;
            double zCoord = 999.0f;
            if (floor_double > floor_double4) {
                xCoord = floor_double4 + 1.0f;
            }
            if (floor_double < floor_double4) {
                xCoord = (double)floor_double4;
            }
            if (floor_double2 > floor_double5) {
                yCoord = floor_double5 + 1.0f;
            }
            if (floor_double2 < floor_double5) {
                yCoord = (double)floor_double5;
            }
            if (floor_double3 > floor_double6) {
                zCoord = floor_double6 + 1.0f;
            }
            if (floor_double3 < floor_double6) {
                zCoord = (double)floor_double6;
            }
            double n2 = 999.0f;
            double n3 = 999.0f;
            double n4 = 999.0f;
            final double n5 = vec3D2.xCoord - vec3D.xCoord;
            final double n6 = vec3D2.yCoord - vec3D.yCoord;
            final double n7 = vec3D2.zCoord - vec3D.zCoord;
            if (xCoord != 999.0f) {
                n2 = (xCoord - vec3D.xCoord) / n5;
            }
            if (yCoord != 999.0f) {
                n3 = (yCoord - vec3D.yCoord) / n6;
            }
            if (zCoord != 999.0f) {
                n4 = (zCoord - vec3D.zCoord) / n7;
            }
            int n8;
            if (n2 < n3 && n2 < n4) {
                if (floor_double > floor_double4) {
                    n8 = 4;
                }
                else {
                    n8 = 5;
                }
                vec3D.xCoord = xCoord;
                vec3D.yCoord += n6 * n2;
                vec3D.zCoord += n7 * n2;
            }
            else if (n3 < n4) {
                if (floor_double2 > floor_double5) {
                    n8 = 0;
                }
                else {
                    n8 = 1;
                }
                vec3D.xCoord += n5 * n3;
                vec3D.yCoord = yCoord;
                vec3D.zCoord += n7 * n3;
            }
            else {
                if (floor_double3 > floor_double6) {
                    n8 = 2;
                }
                else {
                    n8 = 3;
                }
                vec3D.xCoord += n5 * n4;
                vec3D.yCoord += n6 * n4;
                vec3D.zCoord = zCoord;
            }
            final Vec3D vec3D4;
            final Vec3D vec3D3 = vec3D4 = new Vec3D(vec3D.xCoord, vec3D.yCoord, vec3D.zCoord);
            final double xCoord2 = (double)MathHelper.floor_double(vec3D.xCoord);
            vec3D3.xCoord = xCoord2;
            floor_double4 = (int)xCoord2;
            if (n8 == 5) {
                --floor_double4;
                final Vec3D vec3D5 = vec3D4;
                ++vec3D5.xCoord;
            }
            final Vec3D vec3D6 = vec3D4;
            final double yCoord2 = (double)MathHelper.floor_double(vec3D.yCoord);
            vec3D6.yCoord = yCoord2;
            floor_double5 = (int)yCoord2;
            if (n8 == 1) {
                --floor_double5;
                final Vec3D vec3D7 = vec3D4;
                ++vec3D7.yCoord;
            }
            final Vec3D vec3D8 = vec3D4;
            final double zCoord2 = (double)MathHelper.floor_double(vec3D.zCoord);
            vec3D8.zCoord = zCoord2;
            floor_double6 = (int)zCoord2;
            if (n8 == 3) {
                --floor_double6;
                final Vec3D vec3D9 = vec3D4;
                ++vec3D9.zCoord;
            }
            final int blockId = this.getBlockId(floor_double4, floor_double5, floor_double6);
            final Block block = Block.blocksList[blockId];
            final MovingObjectPosition collisionRayTrace;
            if (blockId > 0 && block.isCollidable() && (collisionRayTrace = block.collisionRayTrace(this, floor_double4, floor_double5, floor_double6, vec3D, vec3D2)) != null) {
                return collisionRayTrace;
            }
        }
        return null;
    }
    
    public final boolean growTrees(final int integer1, final int integer2, final int integer3) {
        /*final int n = this.random.nextInt(3) + 4;
        int n2 = 1;
        if (integer2 <= 0 || integer2 + n + 1 > this.height) {
            return false;
        }
        for (int i = integer2; i <= integer2 + 1 + n; ++i) {
            int j = 1;
            if (i == integer2) {
                j = 0;
            }
            if (i >= integer2 + 1 + n - 2) {
                j = 2;
            }
            for (int n3 = integer1 - j; n3 <= integer1 + j && n2 != 0; ++n3) {
                for (int n4 = integer3 - j; n4 <= integer3 + j && n2 != 0; ++n4) {
                    if (n3 >= 0 && i >= 0 && n4 >= 0 && n3 < this.width && i < this.height && n4 < this.length) {
                        final int k;
                        if ((k = (this.blocks[(i * this.length + n4) * this.width + n3] & 0xFF)) != 0) {
                            n2 = 0;
                        }
                    }
                    else {
                        n2 = 0;
                    }
                }
            }
        }
        if (n2 == 0) {
            return false;
        }
        int i;
        if (((i = (this.blocks[((integer2 - 1) * this.length + integer3) * this.width + integer1] & 0xFF)) != Block.grass.blockID && i != Block.dirt.blockID) || integer2 >= this.height - n - 1) {
            return false;
        }
        this.setBlockWithNotify(integer1, integer2 - 1, integer3, Block.dirt.blockID);
        for (int j = integer2 - 3 + n; j <= integer2 + n; ++j) {
            final int n3 = j - (integer2 + n);
            for (int n4 = 1 - n3 / 2, k = integer1 - n4; k <= integer1 + n4; ++k) {
                final int n5 = k - integer1;
                for (i = integer3 - n4; i <= integer3 + n4; ++i) {
                    final int n6 = i - integer3;
                    if ((Math.abs(n5) != n4 || Math.abs(n6) != n4 || (this.random.nextInt(2) != 0 && n3 != 0)) && !Block.opaqueCubeLookup[this.getBlockId(k, j, i)]) {
                        this.setBlockWithNotify(k, j, i, Block.leaves.blockID);
                    }
                }
            }
        }
        for (int j = 0; j < n; ++j) {
            if (!Block.opaqueCubeLookup[this.getBlockId(integer1, integer2 + j, integer3)]) {
                this.setBlockWithNotify(integer1, integer2 + j, integer3, Block.wood.blockID);
            }
        }*/
        return true;
    }
    
    public final Entity getPlayer() {
        return this.player;
    }
    
    public final void entityJoinedWorld(final Entity entity) {
        this.blockMap.insert(entity);
        entity.setWorld(this);
        for (int i = 0; i < this.worldAccesses.size(); ++i) {
            ((IWorldAccess)this.worldAccesses.get(i)).obtainEntitySkin(entity);
        }
    }
    
    public final void releaseEntitySkin(final Entity entity) {
        this.blockMap.remove(entity);
        for (int i = 0; i < this.worldAccesses.size(); ++i) {
            ((IWorldAccess)this.worldAccesses.get(i)).releaseEntitySkin(entity);
        }
    }
    
    public final void createExplosion(final Entity entity, final double double2, final double double3, final double double4, double double5) {
        this.playSoundAtPlayer(double2, double3, double4, "random.explode", 4.0f, (1.0f + (this.random.nextDouble() - this.random.nextDouble()) * 0.2f) * 0.7f);
        final ArrayList<BlockPos> set = new ArrayList<BlockPos>();
        double n = double5;
        for (int i = 0; i < 16; ++i) {
            for (int j = 0; j < 16; ++j) {
                for (int k = 0; k < 16; ++k) {
                    if (i == 0 || i == 15 || j == 0 || j == 15 || k == 0 || k == 15) {
                        double sqrt_double = i / 15.0f * 2.0f - 1.0f;
                        double n2 = j / 15.0f * 2.0f - 1.0f;
                        double n3 = k / 15.0f * 2.0f - 1.0f;
                        final double n4 = (double)Math.sqrt((double)(sqrt_double * sqrt_double + n2 * n2 + n3 * n3));
                        sqrt_double /= n4;
                        n2 /= n4;
                        n3 /= n4;
                        double n5 = double5 * (0.7f + this.random.nextDouble() * 0.6f);
                        double n6 = double2;
                        double n7 = double3;
                        double n8 = double4;
                        while (n5 > 0.0f) {
                            final int n9 = (int)n6;
                            final int n10 = (int)n7;
                            final int blockId = (int)n8;
                            final int l;
                            if ((l = this.getBlockId(n9, n10, blockId)) > 0) {
                                n5 -= (Block.blocksList[l].getExplosionResistance() + 0.3f) * 0.3f;
                            }
                            if (n5 > 0.0f) {
                            	BlockPos pos = new BlockPos(n9,n10,blockId);
                                if(!set.contains(pos))
                                	set.add(pos);
                            }
                            n6 += sqrt_double * 0.3f;
                            n7 += n2 * 0.3f;
                            n8 += n3 * 0.3f;
                            n5 -= 0.22500001f;
                        }
                    }
                }
            }
        }
        double5 *= 2.0f;
        int i = (int)(double2 - double5 - 1.0f);
        int j = (int)(double2 + double5 + 1.0f);
        int k = (int)(double3 - double5 - 1.0f);
        final List entities = this.blockMap.getEntities(entity, (double)i, (double)k, (double)(int)(double4 - double5 - 1.0f), (double)j, (double)(int)(double3 + double5 + 1.0f), (double)(int)(double4 + double5 + 1.0f));
        final Vec3D vec3D = new Vec3D(double2, double3, double4);
        for (int n11 = 0; n11 < entities.size(); ++n11) {
            final Entity entity3;
            final Entity entity2 = entity3 = (Entity)entities.get(n11);
            final double sqrt_double = double4;
            double n12 = double3;
            double double6 = double2;
            final Entity entity4 = entity2;
            double6 = entity2.posX - double6;
            n12 = entity4.posY - n12;
            double n13 = entity4.posZ - sqrt_double;
            final double n8;
            if ((n8 = MathHelper.sqrt_double(double6 * double6 + n12 * n12 + n13 * n13) / double5) <= 1.0f) {
                n13 = entity3.posX - double2;
                double n14 = entity3.posY - double3;
                double n15 = entity3.posZ - double4;
                final double sqrt_double2 = MathHelper.sqrt_double(n13 * n13 + n14 * n14 + n15 * n15);
                n13 /= sqrt_double2;
                n14 /= sqrt_double2;
                n15 /= sqrt_double2;
                final double double7 = (1.0f - n8) * this.getBlockDensity(vec3D, entity3.boundingBox);
                entity3.attackEntityFrom(entity, (int)((double7 * double7 + double7) / 2.0f * 8.0f * double5 + 1.0f));
                double6 = double7;
                final Entity entity5 = entity3;
                entity5.motionX += n13 * double6;
                final Entity entity6 = entity3;
                entity6.motionY += n14 * double6;
                final Entity entity7 = entity3;
                entity7.motionZ += n15 * double6;
            }
        }
        double5 = n;
        for (BlockPos pos : set) {
            i = pos.z;
            final int n9 = pos.y;
            final int n10 = pos.x;
            if (true) {
                final int blockId = this.getBlockId(i, n9, n10);
                for (int l = 0; l <= 0; ++l) {
                    final double double7 = i + this.random.nextDouble();
                    final double double6 = n9 + this.random.nextDouble();
                    final double double8 = n10 + this.random.nextDouble();
                    double n17 = double7 - double2;
                    n = double6 - double3;
                    double n12 = double8 - double4;
                    double sqrt_double = MathHelper.sqrt_double(n17 * n17 + n * n + n12 * n12);
                    n17 /= sqrt_double;
                    n /= sqrt_double;
                    n12 /= sqrt_double;
                    sqrt_double = (sqrt_double = 0.5f / (sqrt_double / double5 + 0.1f)) * (this.random.nextDouble() * this.random.nextDouble() + 0.3f);
                    n17 *= sqrt_double;
                    n *= sqrt_double;
                    n12 *= sqrt_double;
                    this.spawnParticle("explode", (double7 + double2) / 2.0f, (double6 + double3) / 2.0f, (double8 + double4) / 2.0f, n17, n, n12);
                    this.spawnParticle("smoke", double7, double6, double8, n17, n, n12);
                }
                if (blockId > 0) {
                    Block.blocksList[blockId].dropBlockAsItemWithChance(this, i, n9, n10, this.getBlockMetadata(i, n9, n10), 0.3f);
                    this.setBlockWithNotify(i, n9, n10, 0);
                    Block.blocksList[blockId].onBlockDestroyedByExplosion(this, i, n9, n10);
                }
            }
        }
    }
    
    private double getBlockDensity(final Vec3D vec3D, final AxisAlignedBB aabb) {
        final double n = 1.0f / ((aabb.maxX - aabb.minX) * 2.0f + 1.0f);
        final double n2 = 1.0f / ((aabb.maxY - aabb.minY) * 2.0f + 1.0f);
        final double n3 = 1.0f / ((aabb.maxZ - aabb.minZ) * 2.0f + 1.0f);
        int n4 = 0;
        int n5 = 0;
        for (double n6 = 0.0f; n6 <= 1.0f; n6 += n) {
            for (double n7 = 0.0f; n7 <= 1.0f; n7 += n2) {
                for (double n8 = 0.0f; n8 <= 1.0f; n8 += n3) {
                    if (this.rayTraceBlocks(new Vec3D(aabb.minX + (aabb.maxX - aabb.minX) * n6, aabb.minY + (aabb.maxY - aabb.minY) * n7, aabb.minZ + (aabb.maxZ - aabb.minZ) * n8), vec3D) == null) {
                        ++n4;
                    }
                    ++n5;
                }
            }
        }
        return n4 / (double)n5;
    }
    
    public final Entity findSubclassOf(final Class class1) {
        for (int i = 0; i < this.blockMap.all.size(); ++i) {
            final Entity entity = (Entity)this.blockMap.all.get(i);
            if (class1.isAssignableFrom(entity.getClass())) {
                return entity;
            }
        }
        return null;
    }
    
    public final int fluidFlowCheck(int integer1, int integer2, int integer3, final int integer4, final int integer5) {
        /*if (integer1 < 0 || integer2 < 0 || integer3 < 0 || integer1 >= this.width || integer2 >= this.height || integer3 >= this.length) {
            return -1;
        }
        final int n = integer1;
        final int n2 = integer3;
        int n3 = ((integer2 << 10) + integer3 << 10) + integer1;
        int i = 0;
        final int[] worldIntArray2 = this.worldIntArray2;
        final int n4 = 0;
        ++i;
        worldIntArray2[n4] = integer1 + (integer3 << 10);
        int n5 = -9999;
        if (integer4 == Block.waterStill.blockID || integer4 == Block.waterMoving.blockID) {
            n5 = Block.waterSource.blockID;
        }
        if (integer4 == Block.lavaStill.blockID || integer4 == Block.lavaMoving.blockID) {
            n5 = Block.lavaSource.blockID;
        }
        int n6;
        boolean b;
        do {
            b = false;
            int n7 = -1;
            n6 = 0;
            if (++World.worldShort == 30000) {
                Arrays.fill(this.blocksShortArray, (short)0);
                World.worldShort = 1;
            }
            while (i > 0) {
                int n8 = this.worldIntArray2[--i];
                if (this.blocksShortArray[n8] != World.worldShort) {
                    int n9;
                    for (integer1 = n8 % 1024, n9 = (integer3 = n8 / 1024) - n2; integer1 > 0 && this.blocksShortArray[n8 - 1] != World.worldShort && (this.blocks[(integer2 * this.length + integer3) * this.width + integer1 - 1] == integer4 || this.blocks[(integer2 * this.length + integer3) * this.width + integer1 - 1] == integer5); --integer1, --n8) {}
                    if (integer1 > 0 && this.blocks[(integer2 * this.length + integer3) * this.width + integer1 - 1] == n5) {
                        b = true;
                    }
                    int n10 = 0;
                    int n11 = 0;
                    int n12 = 0;
                    while (integer1 < this.width && this.blocksShortArray[n8] != World.worldShort && (this.blocks[(integer2 * this.length + integer3) * this.width + integer1] == integer4 || this.blocks[(integer2 * this.length + integer3) * this.width + integer1] == integer5)) {
                        if (integer3 > 0) {
                            byte b2;
                            if ((b2 = this.blocks[(integer2 * this.length + integer3 - 1) * this.width + integer1]) == n5) {
                                b = true;
                            }
                            if ((b2 = (byte)((this.blocksShortArray[n8 - 1024] != World.worldShort && (b2 == integer4 || b2 == integer5)) ? 1 : 0)) != 0 && n10 == 0) {
                                this.worldIntArray2[i++] = n8 - 1024;
                            }
                            n10 = b2;
                        }
                        if (integer3 < this.length - 1) {
                            byte b2;
                            if ((b2 = this.getBlockId(integer1, integer2, integer3)) == n5) {
                                b = true;
                            }
                            if ((b2 = (byte)((this.blocksShortArray[n8 + 1024] != World.worldShort && (b2 == integer4 || b2 == integer5)) ? 1 : 0)) != 0 && n11 == 0) {
                                this.worldIntArray2[i++] = n8 + 1024;
                            }
                            n11 = b2;
                        }
                        if (integer2 < this.height - 1) {
                            byte b2;
                            if ((b2 = (byte)(((b2 = this.blocks[((integer2 + 1) * this.length + integer3) * this.width + integer1]) == integer4 || b2 == integer5) ? 1 : 0)) != 0 && n12 == 0) {
                                this.worldIntArray3[n6++] = n8;
                            }
                            n12 = b2;
                        }
                        int n13;
                        if ((n13 = (n13 = integer1 - n) + n9) > n7) {
                            n7 = n13;
                            n3 = ((integer2 << 10) + integer3 << 10) + integer1;
                        }
                        this.blocksShortArray[n8++] = World.worldShort;
                        ++integer1;
                    }
                    if (integer1 >= this.width || this.blocks[(integer2 * this.length + integer3) * this.width + integer1] != n5) {
                        continue;
                    }
                    b = true;
                }
            }
            ++integer2;
            final int[] worldIntArray3 = this.worldIntArray3;
            this.worldIntArray3 = this.worldIntArray2;
            this.worldIntArray2 = worldIntArray3;
        } while ((i = n6) > 0);
        if (b) {
            return -9999;
        }*/
        return -1;//n3;
    }
    
    public final int unknownFluidMethod(int integer1, final int integer2, int integer3, final int integer4, final int integer5) {
        /*if (integer1 < 0 || integer2 < 0 || integer3 < 0 || integer1 >= this.width || integer2 >= this.height || integer3 >= this.length) {
            return 0;
        }
        if (++World.worldShort == 30000) {
            Arrays.fill(this.blocksShortArray, (short)0);
            World.worldShort = 1;
        }
        int i = 0;
        final int[] worldIntArray2 = this.worldIntArray2;
        final int n = 0;
        ++i;
        worldIntArray2[n] = integer1 + (integer3 << 10);
        while (i > 0) {
            int n2 = this.worldIntArray2[--i];
            if (this.blocksShortArray[n2] != World.worldShort) {
                integer1 = n2 % 1024;
                integer3 = n2 / 1024;
                if (integer1 == 0 || integer1 == this.width - 1 || integer2 == 0 || integer2 == this.height - 1 || integer3 == 0 || integer3 == this.length - 1) {
                    return 2;
                }
                while (integer1 > 0 && this.blocksShortArray[n2 - 1] != World.worldShort && (this.blocks[(integer2 * this.length + integer3) * this.width + integer1 - 1] == integer4 || this.blocks[(integer2 * this.length + integer3) * this.width + integer1 - 1] == integer5)) {
                    --integer1;
                    --n2;
                }
                if (integer1 > 0 && this.blocks[(integer2 * this.length + integer3) * this.width + integer1 - 1] == 0) {
                    return 0;
                }
                int n3 = 0;
                int n4 = 0;
                while (integer1 < this.width && this.blocksShortArray[n2] != World.worldShort && (this.blocks[(integer2 * this.length + integer3) * this.width + integer1] == integer4 || this.blocks[(integer2 * this.length + integer3) * this.width + integer1] == integer5)) {
                    if (integer1 == 0 || integer1 == this.width - 1) {
                        return 2;
                    }
                    if (integer3 > 0) {
                        byte b;
                        if ((b = this.blocks[(integer2 * this.length + integer3 - 1) * this.width + integer1]) == 0) {
                            return 0;
                        }
                        if ((b = (byte)((this.blocksShortArray[n2 - 1024] != World.worldShort && (b == integer4 || b == integer5)) ? 1 : 0)) != 0 && n3 == 0) {
                            this.worldIntArray2[i++] = n2 - 1024;
                        }
                        n3 = b;
                    }
                    if (integer3 < this.length - 1) {
                        byte b;
                        if ((b = this.blocks[(integer2 * this.length + integer3 + 1) * this.width + integer1]) == 0) {
                            return 0;
                        }
                        if ((b = (byte)((this.blocksShortArray[n2 + 1024] != World.worldShort && (b == integer4 || b == integer5)) ? 1 : 0)) != 0 && n4 == 0) {
                            this.worldIntArray2[i++] = n2 + 1024;
                        }
                        n4 = b;
                    }
                    this.blocksShortArray[n2] = World.worldShort;
                    ++n2;
                    ++integer1;
                }
                if (integer1 < this.width && this.blocks[(integer2 * this.length + integer3) * this.width + integer1] == 0) {
                    return 0;
                }
                continue;
            }
        }*/
        return 1;
    }
    
    public final void playSoundAtEntity(final Entity entity, final String string, final double double3, final double double4) {
        for (int i = 0; i < this.worldAccesses.size(); ++i) {
            double n = 16.0f;
            if (double3 > 1.0f) {
                n = 16.0f * double3;
            }
            if (this.player.getDistanceSqToEntity(entity) < n * n) {
                ((IWorldAccess)this.worldAccesses.get(i)).playSound(string, entity.posX, entity.posY - entity.yOffset, entity.posZ, double3, double4);
            }
        }
    }
    
    public final void playSoundEffect(final double double1, final double double2, final double double3, final String string, final double double5) {
        try {
            for (int i = 0; i < this.worldAccesses.size(); ++i) {
                ((IWorldAccess)this.worldAccesses.get(i)).playMusic(string, double1, double2, double3, 0.0f);
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public final void playSoundAtPlayer(final double double1, final double double2, final double double3, final String string, final double double5, final double double6) {
        try {
            for (int i = 0; i < this.worldAccesses.size(); ++i) {
                double n = 16.0f;
                if (double5 > 1.0f) {
                    n = 16.0f * double5;
                }
                final double n2 = double1 - this.player.posX;
                final double n3 = double2 - this.player.posY;
                final double n4 = double3 - this.player.posZ;
                if (n2 * n2 + n3 * n3 + n4 * n4 < n * n) {
                    ((IWorldAccess)this.worldAccesses.get(i)).playSound(string, double1, double2, double3, double5, double6);
                }
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public final void extinguishFire(int integer1, int integer2, int integer3, final int integer4) {
        if (integer4 == 0) {
            --integer2;
        }
        if (integer4 == 1) {
            ++integer2;
        }
        if (integer4 == 2) {
            --integer3;
        }
        if (integer4 == 3) {
            ++integer3;
        }
        if (integer4 == 4) {
            --integer1;
        }
        if (integer4 == 5) {
            ++integer1;
        }
        if (this.getBlockId(integer1, integer2, integer3) == Block.fire.blockID) {
            this.playSoundAtPlayer(integer1 + 0.5f, integer2 + 0.5f, integer3 + 0.5f, "random.fizz", 0.5f, 2.6f + (this.random.nextDouble() - this.random.nextDouble()) * 0.8f);
            this.setBlockWithNotify(integer1, integer2, integer3, 0);
        }
    }
    
    public final void setBlockTileEntity(final int integer1, final int integer2, final int integer3, final TileEntity tileEntity) {
        tileEntity.worldObj = this;
        tileEntity.xCoord = integer1;
        tileEntity.yCoord = integer2;
        tileEntity.zCoord = integer3;
        this.map.put((integer1 + (integer2 << 10) + (integer3 << 10 << 10)), tileEntity);
        this.list.add(tileEntity);
    }
    
    public final void removeBlockTileEntity(final int integer1, final int integer2, final int integer3) {
        this.list.remove(this.map.remove((integer1 + (integer2 << 10) + (integer3 << 10 << 10))));
    }
    
    public final TileEntity getBlockTileEntity(final int integer1, final int integer2, final int integer3) {
        final int n = integer1 + (integer2 << 10) + (integer3 << 10 << 10);
        TileEntity tileEntity;
        if ((tileEntity = (TileEntity)this.map.get(n)) == null) {
            ((BlockContainer)Block.blocksList[this.getBlockId(integer1, integer2, integer3)]).onBlockAdded(this, integer1, integer2, integer3);
            tileEntity = (TileEntity)this.map.get(n);
        }
        return tileEntity;
    }
    
    public final void spawnParticle(final String string, final double double2, final double double3, final double double4, final double double5, final double double6, final double double7) {
        for (int i = 0; i < this.worldAccesses.size(); ++i) {
            ((IWorldAccess)this.worldAccesses.get(i)).spawnParticle(string, double2, double3, double4, double5, double6, double7);
        }
    }
    
    public final void randomDisplayUpdates(final int integer1, final int integer2, final int integer3) {
        for (int i = 0; i < 1000; ++i) {
            final int n = integer1 + this.random.nextInt(16) - this.random.nextInt(16);
            final int n2 = integer2 + this.random.nextInt(16) - this.random.nextInt(16);
            final int n3 = integer3 + this.random.nextInt(16) - this.random.nextInt(16);
            final int blockId;
            if ((blockId = this.getBlockId(n, n2, n3)) > 0) {
                Block.blocksList[blockId].randomDisplayTick(this, n, n2, n3, this.rand);
            }
        }
    }
    
    public final String debugSkylightUpdates() {
        return new StringBuilder().append("").append(this.tickList.size()).append(". L: ").append(this.worldClient.skylightUpdatesGUI()).toString();
    }
    
    public final void worldMethod() {
        for (int i = 0; i < this.worldAccesses.size(); ++i) {
            final IWorldAccess worldAccess = (IWorldAccess)this.worldAccesses.get(i);
            for (int j = 0; j < this.blockMap.all.size(); ++j) {
                worldAccess.releaseEntitySkin((Entity)this.blockMap.all.get(i));
            }
        }
    }
    
    private void updateChunkLight(final int integer) {
        this.worldClient.updateDaylightCycle(integer);
    }
    
    public final boolean canBlockSeeTheSky(final int integer1, int integer2, final int integer3) {
        if (this.worldIntArray1[integer1 + integer3 * this.width] <= integer2) {
            return true;
        }
        while (integer2 < this.height) {
            if (Block.opaqueCubeLookup[this.getBlockId(integer1, integer2, integer3)]) {
                return false;
            }
            ++integer2;
        }
        return true;
    }
    
    static {
        World.lightBrightnessTable = new double[16];
        for (int i = 0; i <= 15; ++i) {
            final double n = 1.0f - i / 15.0f;
            World.lightBrightnessTable[i] = (1.0f - n) / (n * 3.0f + 1.0f) * 0.95f + 0.05f;
        }
        World.worldShort = 0;
    }
}

class BlockPos {
	public final int x,y,z;
	public BlockPos(int z, int y, int x) {
		this.x = x;
		this.y = y;
		this.z = z;
	}
	@Override
	public int hashCode() {
		return Objects.hash(x, y, z);
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof BlockPos)) {
			return false;
		}
		BlockPos other = (BlockPos) obj;
		return x == other.x && y == other.y && z == other.z;
	}
}
