package net.minecraft.game.level.generator;

import java.util.Random;

import net.minecraft.game.block.Block;
import net.minecraft.game.level.Chunk;
import net.minecraft.game.level.generator.noise.NoiseGeneratorCombined;
import net.minecraft.game.level.generator.noise.NoiseGeneratorOctaves;

public class ChunkGen {

	public static final int HEIGHT = 64;
	private long seed;
	private Random rand;
	private NoiseGeneratorCombined noise1;
	private NoiseGeneratorCombined noise2;
	private NoiseGeneratorOctaves noise3;
	private NoiseGeneratorOctaves noise4; // never use, island mode only
	private NoiseGeneratorCombined noise5;
	private NoiseGeneratorCombined noise6;
	private NoiseGeneratorOctaves noise7;
	private NoiseGeneratorOctaves noise8;
	
	public ChunkGen() {
		reset(System.nanoTime());
	}
	
	public void reset(long seed) {
		this.seed = seed;
		rand = new Random(seed);
		noise1 = new NoiseGeneratorCombined(new NoiseGeneratorOctaves(rand,8),new NoiseGeneratorOctaves(rand,8));
		noise2 = new NoiseGeneratorCombined(new NoiseGeneratorOctaves(rand,8),new NoiseGeneratorOctaves(rand,8));
		noise3 = new NoiseGeneratorOctaves(rand, 6);
		noise4 = new NoiseGeneratorOctaves(rand, 2);
		noise5 = new NoiseGeneratorCombined(new NoiseGeneratorOctaves(rand,8),new NoiseGeneratorOctaves(rand,8));
		noise6 = new NoiseGeneratorCombined(new NoiseGeneratorOctaves(rand,8),new NoiseGeneratorOctaves(rand,8));
		noise7 = new NoiseGeneratorOctaves(rand,8);
		noise8 = new NoiseGeneratorOctaves(rand,8);
	}
	
	public long getSeed() {
		return seed;
	}

	public void setSeed(long seed) {
		reset(seed);
	}
	

	public Chunk[] provideChunk(long chunkX, long chunkZ) {
		final int waterlevel = 32;
		Chunk[] chunks = new Chunk[HEIGHT / Chunk.HEIGHT];
		byte[][][][] data = new byte[chunks.length][Chunk.WIDTH][Chunk.HEIGHT][Chunk.WIDTH];
		for(int i=0;i<chunks.length;i++)
			chunks[i] = new Chunk(data[i]);
		long posX = chunkX * Chunk.WIDTH, posZ = chunkZ * Chunk.WIDTH;
		for(int x = 0;x < Chunk.WIDTH;x++) {
			for(int z = 0;z < Chunk.WIDTH;z++) {
				double n2 = noise1.NoiseGenerator((posX+x) * 1.3f, (posZ+z) * 2.3f) / 6.0 - 4.0; 
				double n3 = noise2.NoiseGenerator((posX+x) * 1.3f, (posZ+z) * 1.3f) / 5.0 + 10.0 - 4.0; 
				if ((noise3.NoiseGenerator(posX+x, posZ+z) / 8.0) > 0.0) {
                    n3 = n2;
                }
				double n5 = Math.max(n2, n3) / 2.0;
				if (n5 < 0.0) {
                    n5 *= 0.8;
                }
//				System.out.println(n5);
				int height = (int) n5;
				double n8 = noise5.NoiseGenerator((posX+x) << 1, (posZ+z) << 1) / 8.0;
				int n9 = (noise6.NoiseGenerator((posX+x) << 1, (posZ+z) << 1) > 0.0) ? 1 : 0;
				if (n8 > 2.0) {
                    int n10 = (((n10 = height) - n9) / 2 << 1) + n9;
                    height = n10;
                }
				double max = 1;
				final double n4;
				int height3;
                if ((height3 = (int)((height3 = (int)(Math.sqrt(Math.abs(n4 = noise8.NoiseGenerator((posX+x) * 2.3, (posZ+z) * 2.3) / 24.0)) * Math.signum(n4) * 20.0) + waterlevel) * (1.0 - max) + max * HEIGHT)) > waterlevel) {
                    height3 = HEIGHT;
                }
				height += waterlevel;
				int dirtLevel = height;
				height += noise7.NoiseGenerator((posX+x), (posZ+z)) / 24 - 4;
				for(int i=0;i<chunks.length;i++) 
					for(int j=0;j<Chunk.HEIGHT;j++) {
						int y = i * Chunk.HEIGHT + j;
						int block = 0;
						if(y<=dirtLevel) {
							block = Block.dirt.blockID;
						}
						if(y<=height) {
							block = Block.stone.blockID;
						}
						if(dirtLevel==y&&y>=waterlevel) {
							block = Block.grass.blockID;
						}
						if(y<=waterlevel&&block==0) {
							block = Block.waterStill.blockID;
						}
						data[i][x][j][z] = (byte) block;
					}
			}
		}
		
		return chunks;
	}
}
